//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices ;

namespace Bwg.Audio.Acm
{
    unsafe public class AcmInterface
    {
        #region structures

        // This structure describes the details of a driver
        [StructLayout(LayoutKind.Explicit)]
        public struct ACMDRIVERDETAILS
        {
            [FieldOffset(0)]
            public uint cbStruct;
            [FieldOffset(4)]
            public fixed byte fccType[4];
            [FieldOffset(8)]
            public fixed byte fccComp[4];
            [FieldOffset(12)]
            public ushort wMid;
            [FieldOffset(14)]
            public ushort wPid;
            [FieldOffset(16)]
            public uint vdwACM;
            [FieldOffset(20)]
            public uint vdwDriver;
            [FieldOffset(24)]
            public uint fdwSupport;
            [FieldOffset(28)]
            public uint cFormatTags;
            [FieldOffset(32)]
            public uint cFilterTags;
            [FieldOffset(36)]
            public uint hIcon;
            [FieldOffset(40)]
            public fixed byte szShortName[32];
            [FieldOffset(72)]
            public fixed byte szLongName[128];
            [FieldOffset(200)]
            public fixed byte szCopyright[80];
            [FieldOffset(280)]
            public fixed byte szLicensing[128];
            [FieldOffset(408)]
            public fixed byte szFeatures[512];
        } ;

        [StructLayout(LayoutKind.Explicit)]
        public struct WAVEFORMATEX
        {
            [FieldOffset(0)]
            public ushort wFormatTag;
            [FieldOffset(2)]
            public ushort nChannels;
            [FieldOffset(4)]
            public uint nSamplesPerSec;
            [FieldOffset(8)]
            public uint nAvgBytesPerSec;
            [FieldOffset(12)]
            public ushort nBlockAlign;
            [FieldOffset(14)]
            public ushort wBitsPerSample;
            [FieldOffset(16)]
            public ushort cbSize;
        } ;

        [StructLayout(LayoutKind.Explicit)]
        public struct MPEGLAYER3WAVEFORMAT
        {
            [FieldOffset(0)]
            public WAVEFORMATEX wfx;
            [FieldOffset(18)]
            public ushort wID;
            [FieldOffset(20)]
            public uint fdwFlags;
            [FieldOffset(24)]
            public ushort nBlockSize;
            [FieldOffset(26)]
            public ushort nFramesPerBlock;
            [FieldOffset(28)]
            public ushort nCodecDelay;
        } ;

        [StructLayout(LayoutKind.Explicit)]
        public struct MSAUDIO1WAVEFORMAT
        {
            [FieldOffset(0)]
            public WAVEFORMATEX wfx;
            [FieldOffset(18)]
            public fixed ushort extra[5];
        } ;

        [StructLayout(LayoutKind.Explicit)]
        public struct ACMSTREAMHEADER
        {
            [FieldOffset(0)]
            public uint cbStruct ;
            [FieldOffset(4)]
            public uint fdwStatus ;
            [FieldOffset(8)]
            public uint dwUser ;
            [FieldOffset(12)]
            public IntPtr pbSrc ;
            [FieldOffset(16)]
            public uint cbSrcLength ;
            [FieldOffset(20)]
            public uint cbSrcLengthUsed ;
            [FieldOffset(24)]
            public uint dwSrcUser ;
            [FieldOffset(28)]
            public IntPtr pbDst ;
            [FieldOffset(32)]
            public uint cbDstLength ;
            [FieldOffset(36)]
            public uint cbDstLengthUsed ;
            [FieldOffset(40)]
            public uint dwDstUser ;
            [FieldOffset(44)]
            public fixed uint dwReservedDriver[10] ;
        } ;

        [StructLayout(LayoutKind.Explicit)]
        public struct WAVEFILTER
        {
            [FieldOffset(0)]
            public uint cbStruct;
            [FieldOffset(4)]
            public uint dwFilterTag;
            [FieldOffset(8)]
            public uint fdwFilter;
            [FieldOffset(12)]
            public fixed uint fdwSupport[5] ;
        } ;

        [StructLayout(LayoutKind.Explicit)]
        internal struct ACMFILTERDETAILS
        {
            [FieldOffset(0)]
            public uint cbStruct;
            [FieldOffset(4)]
            public uint dwFilterIndex;
            [FieldOffset(8)]
            public uint dwFilterTag;
            [FieldOffset(12)]
            public uint fdwSupport;
            [FieldOffset(16)]
            public WAVEFILTER* pwfltr;
            [FieldOffset(20)]
            public uint cbwfltr;
            [FieldOffset(24)]
            public fixed byte szFilter[128];
        } ;

        [StructLayout(LayoutKind.Explicit)]
        internal struct ACMFORMATDETAILS
        {
            [FieldOffset(0)]
            public uint cbStruct ;
            [FieldOffset(4)]
            public uint dwFormatIndex ;
            [FieldOffset(8)]
            public uint dwFormatTag ;
            [FieldOffset(12)]
            public uint fdwSupport ;
            [FieldOffset(16)]
            public WAVEFORMATEX *pwfx ;
            [FieldOffset(20)]
            public uint cbwfx ;
            [FieldOffset(24)]
            public fixed byte szFormat[128] ;
        } ;

        [StructLayout(LayoutKind.Explicit)]
        internal struct ACMFORMATTAGDETAILS
        {
            [FieldOffset(0)]
            public uint cbStruct;
            [FieldOffset(4)]
            public uint dwFormatTagIndex;
            [FieldOffset(8)]
            public uint dwFormatTag;
            [FieldOffset(12)]
            public uint cbFormatSize;
            [FieldOffset(16)]
            public uint fdwSupport;
            [FieldOffset(20)]
            public uint cStandardFormats;
            [FieldOffset(24)]
            public fixed byte szFormatTag[48];
        } ;

        #endregion

        #region delegates
        internal delegate bool AcmDriverEnumCallback(uint driverid, uint dwInstance, uint dwSupport) ;
        internal delegate bool AcmFilterEnumCallback(uint driverid, ACMFILTERDETAILS* details_p, uint instance, uint support);
        internal delegate bool AcmFormatEnumCallback(uint driverid, ACMFORMATDETAILS* details_p, uint instance, uint support);
        internal delegate bool AcmFormatTagEnumCallback(uint driverid, ACMFORMATTAGDETAILS* details_p, uint instance, uint support);
        #endregion

        #region external functions
        [DllImport("Msacm32.dll")]
        internal static extern uint acmGetVersion();

        [DllImport("Msacm32.dll")]
        internal static extern uint acmStreamOpen(uint* handle, uint driver_handle, byte* src,
                                                  byte* dest, IntPtr filter, uint callback,
                                                  uint instance, uint flags);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmDriverID(uint obj, uint* id, uint flags);

        [DllImport("Msacm32.dll", CharSet = CharSet.Ansi)]
        internal static extern uint acmDriverDetails(uint id, ACMDRIVERDETAILS* detail_p, uint details);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmStreamSize(uint h, uint insize, uint* outsize, uint flags);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmStreamPrepareHeader(uint h, ACMSTREAMHEADER *hdr, uint flags);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmStreamConvert(uint h, ACMSTREAMHEADER *hdr, uint flags);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmStreamUnprepareHeader(uint h, ACMSTREAMHEADER* hdr, uint flags);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmStreamClose(uint h, uint flags);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmDriverEnum(AcmDriverEnumCallback cb, uint dwInstance, uint dwEnum) ;

        [DllImport("Msacm32.dll", CharSet = CharSet.Ansi)]
        internal static extern uint acmFilterEnum(uint drvhandle, ACMFILTERDETAILS* detail_p, AcmFilterEnumCallback cb, 
                                                  uint instance, uint denum);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmFormatEnum(uint drvhandle, ACMFORMATDETAILS *detail_p, AcmFormatEnumCallback cb,
                                                  uint instance, uint denum) ;

        [DllImport("Msacm32.dll", CharSet = CharSet.Ansi)]
        internal static extern uint acmFormatTagEnum(uint drvid, ACMFORMATTAGDETAILS *detail_p, AcmFormatTagEnumCallback cb,
                                                     uint instance, uint denum) ;

        [DllImport("Msacm32.dll")]
        internal static extern uint acmDriverOpen(uint* handle, uint driverid, uint fdwOpen);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmDriverClose(uint handle, uint fdwOpen);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmFormatDetails(uint handle, ACMFORMATDETAILS* details_p, uint details);

        [DllImport("Msacm32.dll")]
        internal static extern uint acmMetrics(uint obj, uint metric, uint* value);

        [DllImport("ntdll.dll")]
        internal static extern void RtlMoveMemory(IntPtr dest, IntPtr src, int size) ;

        #endregion

        #region constants

        public const uint ACM_FORMATDETAILSF_FORMAT = 0x00000001;
        public const uint ACM_FORMATDETAILSF_INDEX = 0x00000000;

        public const ushort MPEGLAYER3_WFX_EXTRA_BYTES = 12;

        public const ushort MPEGLAYER3_ID_MPEG = 1;
        public const uint MPEGLAYER3_FLAG_PADDING_OFF = 0x00000002;

        public const uint ACM_STREAMCONVERTF_BLOCKALIGN = 0x00000004;
        public const uint ACM_STREAMCONVERTF_START = 0x00000010;
        public const uint ACM_STREAMCONVERTF_END = 0x00000020;

        public const uint ACM_STREAMSIZEF_DESTINATION = 0x00000001;
        public const uint ACM_STREAMSIZEF_SOURCE = 0x00000000;

        // Driver support flags
        public const uint ACMDRIVERDETAILS_SUPPORTF_ASYNC = 0x00000010;
        public const uint ACMDRIVERDETAILS_SUPPORTF_CODEC = 0x00000001;
        public const uint ACMDRIVERDETAILS_SUPPORTF_CONVERTER = 0x00000002;
        public const uint ACMDRIVERDETAILS_SUPPORTF_DISABLED = 0x80000000;
        public const uint ACMDRIVERDETAILS_SUPPORTF_FILTER = 0x00000004;
        public const uint ACMDRIVERDETAILS_SUPPORTF_HARDWARE = 0x00000008; 
        public const uint ACMDRIVERDETAILS_SUPPORTF_LOCAL = 0x40000000;

        // Wave format tags
        public enum WaveFormat : ushort
        {
            WAVE_FORMAT_UNKNOWN = 0x0000,
            WAVE_FORMAT_PCM = 0x0001,
            WAVE_FORMAT_ADPCM = 0x0002,
            WAVE_FORMAT_IEEE_FLOAT = 0x0003,
            WAVE_FORMAT_VSELP = 0x0004,
            WAVE_FORMAT_IBM_CVSD = 0x0005,
            WAVE_FORMAT_ALAW = 0x0006,
            WAVE_FORMAT_MULAW = 0x0007,
            WAVE_FORMAT_DTS = 0x0008,
            WAVE_FORMAT_DRM = 0x0009,
            WAVE_FORMAT_OKI_ADPCM = 0x0010,
            WAVE_FORMAT_DVI_ADPCM = 0x0011,
            WAVE_FORMAT_IMA_ADPCM = 0x0011,
            WAVE_FORMAT_MEDIASPACE_ADPCM = 0x0012,
            WAVE_FORMAT_SIERRA_ADPCM = 0x0013,
            WAVE_FORMAT_G723_ADPCM = 0x0014,
            WAVE_FORMAT_DIGISTD = 0x0015,
            WAVE_FORMAT_DIGIFIX = 0x0016,
            WAVE_FORMAT_DIALOGIC_OKI_ADPCM = 0x0017,
            WAVE_FORMAT_MEDIAVISION_ADPCM = 0x0018,
            WAVE_FORMAT_CU_CODEC = 0x0019,
            WAVE_FORMAT_YAMAHA_ADPCM = 0x0020,
            WAVE_FORMAT_SONARC = 0x0021,
            WAVE_FORMAT_DSPGROUP_TRUESPEECH = 0x0022,
            WAVE_FORMAT_ECHOSC1 = 0x0023,
            WAVE_FORMAT_AUDIOFILE_AF36 = 0x0024,
            WAVE_FORMAT_APTX = 0x0025,
            WAVE_FORMAT_AUDIOFILE_AF10 = 0x0026,
            WAVE_FORMAT_PROSODY_1612 = 0x0027,
            WAVE_FORMAT_LRC = 0x0028,
            WAVE_FORMAT_DOLBY_AC2 = 0x0030,
            WAVE_FORMAT_GSM610 = 0x0031,
            WAVE_FORMAT_MSNAUDIO = 0x0032,
            WAVE_FORMAT_ANTEX_ADPCME = 0x0033,
            WAVE_FORMAT_CONTROL_RES_VQLPC = 0x0034,
            WAVE_FORMAT_DIGIREAL = 0x0035,
            WAVE_FORMAT_DIGIADPCM = 0x0036,
            WAVE_FORMAT_CONTROL_RES_CR10 = 0x0037,
            WAVE_FORMAT_NMS_VBXADPCM = 0x0038,
            WAVE_FORMAT_CS_IMAADPCM = 0x0039,
            WAVE_FORMAT_ECHOSC3 = 0x003A,
            WAVE_FORMAT_ROCKWELL_ADPCM = 0x003B,
            WAVE_FORMAT_ROCKWELL_DIGITALK = 0x003C,
            WAVE_FORMAT_XEBEC = 0x003D,
            WAVE_FORMAT_G721_ADPCM = 0x0040,
            WAVE_FORMAT_G728_CELP = 0x0041,
            WAVE_FORMAT_MSG723 = 0x0042,
            WAVE_FORMAT_MPEG = 0x0050,
            WAVE_FORMAT_RT24 = 0x0052,
            WAVE_FORMAT_PAC = 0x0053,
            WAVE_FORMAT_MPEGLAYER3 = 0x0055,
            WAVE_FORMAT_LUCENT_G723 = 0x0059,
            WAVE_FORMAT_CIRRUS = 0x0060,
            WAVE_FORMAT_ESPCM = 0x0061,
            WAVE_FORMAT_VOXWARE = 0x0062,
            WAVE_FORMAT_CANOPUS_ATRAC = 0x0063,
            WAVE_FORMAT_G726_ADPCM = 0x0064,
            WAVE_FORMAT_G722_ADPCM = 0x0065,
            WAVE_FORMAT_DSAT_DISPLAY = 0x0067,
            WAVE_FORMAT_VOXWARE_BYTE_ALIGNED = 0x0069,
            WAVE_FORMAT_VOXWARE_AC8 = 0x0070,
            WAVE_FORMAT_VOXWARE_AC10 = 0x0071,
            WAVE_FORMAT_VOXWARE_AC16 = 0x0072,
            WAVE_FORMAT_VOXWARE_AC20 = 0x0073,
            WAVE_FORMAT_VOXWARE_RT24 = 0x0074,
            WAVE_FORMAT_VOXWARE_RT29 = 0x0075,
            WAVE_FORMAT_VOXWARE_RT29HW = 0x0076,
            WAVE_FORMAT_VOXWARE_VR12 = 0x0077,
            WAVE_FORMAT_VOXWARE_VR18 = 0x0078,
            WAVE_FORMAT_VOXWARE_TQ40 = 0x0079,
            WAVE_FORMAT_SOFTSOUND = 0x0080,
            WAVE_FORMAT_VOXWARE_TQ60 = 0x0081,
            WAVE_FORMAT_MSRT24 = 0x0082,
            WAVE_FORMAT_G729A = 0x0083,
            WAVE_FORMAT_MVI_MVI2 = 0x0084,
            WAVE_FORMAT_DF_G726 = 0x0085,
            WAVE_FORMAT_DF_GSM610 = 0x0086,
            WAVE_FORMAT_ISIAUDIO = 0x0088,
            WAVE_FORMAT_ONLIVE = 0x0089,
            WAVE_FORMAT_SBC24 = 0x0091,
            WAVE_FORMAT_DOLBY_AC3_SPDIF = 0x0092,
            WAVE_FORMAT_MEDIASONIC_G723 = 0x0093,
            WAVE_FORMAT_PROSODY_8KBPS = 0x0094,
            WAVE_FORMAT_ZYXEL_ADPCM = 0x0097,
            WAVE_FORMAT_PHILIPS_LPCBB = 0x0098,
            WAVE_FORMAT_PACKED = 0x0099,
            WAVE_FORMAT_MALDEN_PHONYTALK = 0x00A0,
            WAVE_FORMAT_RHETOREX_ADPCM = 0x0100,
            WAVE_FORMAT_IRAT = 0x0101,
            WAVE_FORMAT_VIVO_G723 = 0x0111,
            WAVE_FORMAT_VIVO_SIREN = 0x0112,
            WAVE_FORMAT_DIGITAL_G723 = 0x0123,
            WAVE_FORMAT_SANYO_LD_ADPCM = 0x0125,
            WAVE_FORMAT_SIPROLAB_ACEPLNET = 0x0130,
            WAVE_FORMAT_SIPROLAB_ACELP4800 = 0x0131,
            WAVE_FORMAT_SIPROLAB_ACELP8V3 = 0x0132,
            WAVE_FORMAT_SIPROLAB_G729 = 0x0133,
            WAVE_FORMAT_SIPROLAB_G729A = 0x0134,
            WAVE_FORMAT_SIPROLAB_KELVIN = 0x0135,
            WAVE_FORMAT_G726ADPCM = 0x0140,
            WAVE_FORMAT_QUALCOMM_PUREVOICE = 0x0150,
            WAVE_FORMAT_QUALCOMM_HALFRATE = 0x0151,
            WAVE_FORMAT_TUBGSM = 0x0155,
            WAVE_FORMAT_MSAUDIO1 = 0x0160,
            WAVE_FORMAT_MSAUDIO2 = 0x0161,
            WAVE_FORMAT_UNISYS_NAP_ADPCM = 0x0170,
            WAVE_FORMAT_UNISYS_NAP_ULAW = 0x0171,
            WAVE_FORMAT_UNISYS_NAP_ALAW = 0x0172,
            WAVE_FORMAT_UNISYS_NAP_16K = 0x0173,
            WAVE_FORMAT_CREATIVE_ADPCM = 0x0200,
            WAVE_FORMAT_CREATIVE_FASTSPEECH8 = 0x0202,
            WAVE_FORMAT_CREATIVE_FASTSPEECH10 = 0x0203,
            WAVE_FORMAT_UHER_ADPCM = 0x0210,
            WAVE_FORMAT_QUARTERDECK = 0x0220,
            WAVE_FORMAT_ILINK_VC = 0x0230,
            WAVE_FORMAT_RAW_SPORT = 0x0240,
            WAVE_FORMAT_ESST_AC3 = 0x0241,
            WAVE_FORMAT_IPI_HSX = 0x0250,
            WAVE_FORMAT_IPI_RPELP = 0x0251,
            WAVE_FORMAT_CS2 = 0x0260,
            WAVE_FORMAT_SONY_SCX = 0x0270,
            WAVE_FORMAT_FM_TOWNS_SND = 0x0300,
            WAVE_FORMAT_BTV_DIGITAL = 0x0400,
            WAVE_FORMAT_QDESIGN_MUSIC = 0x0450,
            WAVE_FORMAT_VME_VMPCM = 0x0680,
            WAVE_FORMAT_TPC = 0x0681,
            WAVE_FORMAT_OLIGSM = 0x1000,
            WAVE_FORMAT_OLIADPCM = 0x1001,
            WAVE_FORMAT_OLICELP = 0x1002,
            WAVE_FORMAT_OLISBC = 0x1003,
            WAVE_FORMAT_OLIOPR = 0x1004,
            WAVE_FORMAT_LH_CODEC = 0x1100,
            WAVE_FORMAT_NORRIS = 0x1400,
            WAVE_FORMAT_SOUNDSPACE_MUSICOMPRESS = 0x1500,
            WAVE_FORMAT_DVM = 0x2000,
        } ;

        // Metric values for acmMetrics()
        public const uint ACM_METRIC_COUNT_DRIVERS = 1 ;
        public const uint ACM_METRIC_COUNT_CODECS = 2 ;
        public const uint ACM_METRIC_COUNT_CONVERTERS = 3 ;
        public const uint ACM_METRIC_COUNT_FILTERS = 4 ;
        public const uint ACM_METRIC_COUNT_DISABLED = 5 ;
        public const uint ACM_METRIC_COUNT_HARDWARE = 6 ;
        public const uint ACM_METRIC_COUNT_LOCAL_DRIVERS = 20;
        public const uint ACM_METRIC_COUNT_LOCAL_CODECS = 21;
        public const uint ACM_METRIC_COUNT_LOCAL_CONVERTERS = 22;
        public const uint ACM_METRIC_COUNT_LOCAL_FILTERS = 23;
        public const uint ACM_METRIC_COUNT_LOCAL_DISABLED = 24;
        public const uint ACM_METRIC_HARDWARE_WAVE_INPUT = 30;
        public const uint ACM_METRIC_HARDWARE_WAVE_OUTPUT = 31;
        public const uint ACM_METRIC_MAX_SIZE_FORMAT = 50;
        public const uint ACM_METRIC_MAX_SIZE_FILTER = 51;
        public const uint ACM_METRIC_DRIVER_SUPPORT  = 100;
        public const uint ACM_METRIC_DRIVER_PRIORITY = 101;

        #endregion
    }
}
