//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Bwg.Logging ;
using Bwg.Scsi;
using Bwg.Audio;
using Bwg.Audio.Acm;

namespace Bwg.Audio.Ripper
{
    public unsafe class CDRipper
    {

        public ProgressChangedEventHandler Progress;

        private AudioManager m_mgr;
        private Device m_device;

        public CDRipper(Device dev, Logger l)
        {
            m_device = dev;
            m_mgr = new AudioManager(l);
        }

        public int LastTrackNumber
        {
            get
            {
                IList<TocEntry> toc;
                m_device.ReadToc(0, true, out toc);
                return toc.Count - 1;
            }
        }

        public Device.CommandStatus RipTrack(int track, string filename, IDictionary<string, object> props, 
                                             AcmDriver drv, AcmFormatDetails fmt)
        {
            IList<TocEntry> toc;
            Device.CommandStatus st;

            st = m_device.SetCdSpeed(Device.RotationalControl.CLVandNonPureCav, Device.OptimumSpeed, Device.OptimumSpeed);
            if (st != Device.CommandStatus.Success)
                return st;

            st = m_device.ReadToc((byte)track, false, out toc);
            if (st != Device.CommandStatus.Success)
                return st ;

            TrackInformation tinfo;
            st = m_device.ReadTrackInformation(Device.ReadTrackType.TrackNumber, (uint)track, out tinfo);
            if (st != Device.CommandStatus.Success)
                return st;

            TocEntry entry = toc[0];
            int tsize = (int)tinfo.TrackSize ;

            CDAudioReader rdr = new CDAudioReader(m_device, (int)entry.StartSector, tsize);
            rdr.Progress += new ProgressChangedEventHandler(ReadProgress);

            AudioWriter writer = new AcmWriter(m_mgr.AcmManager, props, rdr, drv, fmt);
            writer.Write(filename);

            // Be sure the writer is closed and all data is flushed
            writer.Close();

            rdr.Progress -= new ProgressChangedEventHandler(ReadProgress);

            return Device.CommandStatus.Success;
        }

        private void ReadProgress(object sender, ProgressChangedEventArgs args)
        {
            if (Progress != null)
                Progress(this, args);
        }
    }
}
