//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using System.ComponentModel;
using Bwg.Audio.Acm ;
using Bwg.Logging;

namespace Bwg.Audio
{
    public class AudioManager
    {
        #region private variables
        /// <summary>
        /// This is the ACM manager that manages drivers and associated format
        /// types.
        /// </summary>
        private AcmManager m_acm_manager;

        /// <summary>
        /// The message logger for logging messages
        /// </summary>
        private Logger m_logger;

        private string[] m_extensions;
        #endregion

        #region public properties

        public AcmManager AcmManager
        {
            get
            {
                return m_acm_manager;
            }
        }

        public string[] Extensions
        {
            get
            {
                return m_extensions;
            }
        }
        #endregion

        #region static public vars
        static public ProgressChangedEventHandler ScanProgress;
        #endregion

        #region constructors
        public AudioManager(Logger l)
        {
            m_logger = l;
            m_acm_manager = new AcmManager(l);

            m_extensions = new string[] { "mp3", "wav", "MP3", "WAV", "wma", "WMA" };
        }
        #endregion

        #region public methods

        public bool AudioDriverExists(string ext)
        {
            if (ext.Length == 0)
                return false;

            if (ext[0] == '.')
                ext = ext.Substring(1);
            ext = ext.ToLower();
            return ext == "mp3" || ext == "wav" || ext == "wma";
        }

        /// <summary>
        /// Return a reader for the file based on the extension of the file.  If null is
        /// returned, no reader is available for the file type.
        /// </summary>
        /// <param name="fullpath"></param>
        /// <returns></returns>
        public AudioReader GetReader(string fullpath)
        {
            string ext = Path.GetExtension(fullpath);
            if (ext.Length == 0)
                throw new InvalidFileFormat(fullpath, "no extension on the file, cannot determine reader type");

            if (ext[0] == '.')
                ext = ext.Substring(1);
            ext = ext.ToLower();

            AudioReader rdr = null;
            if (ext == "mp3")
            {
                if (m_acm_manager.SupportsType(AcmInterface.WaveFormat.WAVE_FORMAT_MPEGLAYER3))
                {
                    AudioHeaderReader hdrrdr = new Mp3HeaderReader(m_acm_manager);
                    rdr = new AcmReader(m_acm_manager, fullpath, hdrrdr);
                }
                else
                {
                    throw new Exception("no codec was found that supports the MP3 file type");
                }
            }
            else if (ext == "wav")
            {
                if (m_acm_manager.SupportsType(AcmInterface.WaveFormat.WAVE_FORMAT_PCM))
                {
                    AudioHeaderReader hdrrdr = new WaveHeaderReader(m_acm_manager);
                    rdr = new AcmReader(m_acm_manager, fullpath, hdrrdr);
                }
                else
                {
                    throw new Exception("no codec was found that supports the WAV file type");
                }
            }
            else if (ext == "wma")
            {
                rdr = new WindowsMediaReader(fullpath);
            }

            if (rdr == null)
                throw new InvalidFileFormat(fullpath, "no reader is available to read this file format");

            return rdr;
        }

        #endregion
    }
}
