//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;
using Bwg.Audio.Acm ;

namespace Bwg.Audio
{
    unsafe class WaveHeaderWriter : AudioHeaderWriter
    {
        #region public properties
        public override string Extension
        {
            get
            {
                return "wav";
            }
        }
        #endregion

        #region private constants
        const uint HeaderByteCount = 8;
        #endregion

        #region constructor
        public WaveHeaderWriter(AcmFormatDetails fmt)
            : base(fmt)
        {
            Debug.Assert(fmt.Tag == AcmInterface.WaveFormat.WAVE_FORMAT_PCM);
        }
        #endregion

        #region public methods
        /// <summary>
        /// Write the header for the given file type
        /// </summary>
        /// <param name="strm">the streawm to write to</param>
        /// <returns>true if sucessful, false otherwise</returns>
        public override bool WriteHeader(IDictionary<string, object> props, FileStream strm)
        {
            //
            // Get a typed pointer to the WAVEFORMTEX structure.  We need this info to write the
            // header.
            //
            AcmInterface.WAVEFORMATEX *p = (AcmInterface.WAVEFORMATEX *)m_format.WaveFormatExPtr ;

            WriteBlockHeader(strm, "RIFF", 0);                                  // Write the initial header
            WriteString(strm, "WAVE");                                          // Set the data type to RIFF

            WriteBlockHeader(strm, "fmt ", 16);
            Write16(strm, (UInt16)m_format.Tag) ;
            Write16(strm, p->nChannels);
            Write32(strm, p->nSamplesPerSec);
            Write32(strm, p->nBlockAlign * p->nSamplesPerSec);
            Write16(strm, p->nBlockAlign);
            Write16(strm, p->wBitsPerSample);

            WriteBlockHeader(strm, "data", 0);

            return true;
        }

        /// <summary>
        /// This method patches the previously written header based on the amount
        /// of data written by the converter
        /// </summary>
        /// <param name="strm">the stream to write to</param>
        /// <param name="totaldata">the total amount of audio data written, in bytes</param>
        /// <returns>true if sucessful, false otherwise</returns>
        public override bool PatchHeader(IDictionary<string, object> props, FileStream strm, int totaldata)
        {
            //
            // Get a typed pointer to the WAVEFORMTEX structure.  We need this info to write the
            // header.
            //
            AcmInterface.WAVEFORMATEX* p = (AcmInterface.WAVEFORMATEX*)m_format.WaveFormatExPtr;

            // Magic from the wave format header count
            long total = HeaderByteCount + HeaderByteCount + 4 + 16 + totaldata;
            Debug.Assert(total < Int32.MaxValue);

            strm.Seek(0, System.IO.SeekOrigin.Begin);
            WriteBlockHeader(strm, "RIFF", (uint)total);

            WriteString(strm, "WAVE");                                          // Set the data type to RIFF

            WriteBlockHeader(strm, "fmt ", 16);
            Write16(strm, (UInt16)m_format.Tag);
            Write16(strm, p->nChannels);
            Write32(strm, p->nSamplesPerSec);
            Write32(strm, p->nBlockAlign * p->nSamplesPerSec);
            Write16(strm, p->nBlockAlign);
            Write16(strm, p->wBitsPerSample);

            WriteBlockHeader(strm, "data", (uint)totaldata);

            return true;
        }
        #endregion

        #region private methods
        private void WriteBlockHeader(FileStream strm, string tag, uint cnt)
        {
            Debug.Assert(tag.Length == 4);
            WriteString(strm, tag);
            Write32(strm, cnt);
        }

        #endregion
    }
}
