//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Reflection ;
using System.IO ;
using Bwg.Logging;
using Bwg.Burner;
using Bwg.Scsi;
using Bwg.Controls;

namespace Bwg.Author
{
    public partial class AuthorForm : BwgBurnBaseForm
    {
        #region private member variables
        private IList<TaskButton> m_buttons;
        private HelpTextManager m_help_manager;
        private IDictionary<string, Process> m_processes;
        #endregion

        #region constructor
        /// <summary>
        /// Create the new form for the main Author application
        /// </summary>
        public AuthorForm()
        {
            m_logger = new Logger();
            m_buttons = new List<TaskButton>();

            m_processes = new Dictionary<string, Process>();

            m_help_manager = new HelpTextManager();
            m_help_manager.LoadHelpFile("CDAuthorHelp.xml");

            InitializeComponent();
            HelpButton = true;

            InitializeTaskButtons();

            foreach (TaskButton b in m_buttons)
            {
                if (b.BuddyControl == null)
                    throw new Exception(b.DescriptionText + " button has no buddy control");

                b.HelpRequested += new HelpEventHandler(HelpText_ButtonHelpRequested);
            }
        }
        #endregion

        #region private methods
        private void InitializeTaskButtons()
        {
            // Now, find all TaskButtons
            FindAllTaskButtons();

            // Now add enter and leave handlers to each button
            foreach (TaskButton button in m_buttons)
            {
                button.MouseLeave += new EventHandler(TaskButtonLeave);
                button.MouseEnter += new EventHandler(TaskButtonEnter);
            }
        }

        private void TaskButtonLeave(object sender, EventArgs args)
        {
            if (sender is TaskButton)
            {
                TaskButton b = (TaskButton)sender;
                b.DrawInColor = false;
                HelpText.Text = "";
                b.BuddyControl.ForeColor = Color.LightGray;
                this.Select();
            }
        }

        private void TaskButtonEnter(object sender, EventArgs args)
        {
            if (sender is TaskButton)
            {
                TaskButton b = (TaskButton)sender;
                b.DrawInColor = true;
                HelpText.Text = m_help_manager.GetHelpString(b.DescriptionText);
                b.BuddyControl.ForeColor = Color.Yellow;
            }
        }

        private void FindAllTaskButtons()
        {
            foreach(TabPage p in TabControl.TabPages)
            {
                foreach (Control c in p.Controls)
                {
                    if (c is TaskButton)
                    {
                        TaskButton button = (TaskButton)c;
                        m_buttons.Add(button);
                    }
                }
            }
        }

        private string GetProgramPath(string pgm)
        {
            Assembly a = GetType().Assembly ;
            string dir = Path.GetDirectoryName(a.Location);
            string fname = Path.Combine(dir, pgm) ;
            if (File.Exists(fname))
                return fname;

            string mode = Path.GetFileName(dir);
            dir = Path.GetDirectoryName(dir);
            dir = Path.GetDirectoryName(dir);
            dir = Path.GetDirectoryName(dir);
            dir = Path.Combine(dir, Path.GetFileNameWithoutExtension(pgm));
            dir = Path.Combine(dir, "bin");
            dir = Path.Combine(dir, mode);

            return Path.Combine(dir, pgm);
        }

        private void RunProgram(string pgm)
        {
            RunProgram(pgm, string.Empty);
        }

        private void RunProgram(string pgm, string args)
        {
            if (m_processes.ContainsKey(pgm))
            {
                Process p = m_processes[pgm];
                if (p.HasExited)
                {
                    ProgramExited(p, null);
                    RunProgram(pgm);
                }
                else
                {
                    MessageBox.Show("This program is already running");
                }
            }
            else
            {
                string pgmpath = GetProgramPath(pgm);
                if (!File.Exists(pgmpath))
                {
                    string err = "Cannot find program '" + pgm + "' - please re-install BwgBurn";
                    MessageBox.Show(err, "Error!!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                Process p ;

                if (args != string.Empty)
                    p = Process.Start(pgmpath, args);
                else
                    p = Process.Start(pgmpath);
                p.EnableRaisingEvents = true;
                p.Exited += new EventHandler(ProgramExited);
                m_processes[pgm] = p;
            }
        }

        private void ProgramExited(object sender, EventArgs args)
        {
            string pkey = "";
            foreach (string key in m_processes.Keys)
            {
                if (m_processes[key] == sender)
                {
                    pkey = key;
                    break;
                }
            }

            if (pkey != "")
                m_processes.Remove(pkey);
        }

        private void NotImplemented()
        {
            MessageBox.Show("This function is not yet implemented, stay tuned to http://bwgburn.sourceforge.net for future details.",
                            "Not Implemented", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }
        #endregion

        #region button methods
        private void BurnCDIsoTask_Click(object sender, EventArgs e)
        {
            RunProgram("Bwg.BurnIso.exe") ;
        }

        private void CreateDataCDTask_Click(object sender, EventArgs e)
        {
            RunProgram("Bwg.CreateDataDisk.exe");
        }

        private void EraseDiskTask_Click_1(object sender, EventArgs e)
        {
            RunProgram("Bwg.EraseDisk.exe");
        }

        private void CreateMusicCDTask_Click(object sender, EventArgs e)
        {
            RunProgram("Bwg.CreateAudioDisk.exe");
        }

        private void RipMusicTask_Click(object sender, EventArgs e)
        {
            RunProgram("Bwg.RipAudio.exe");
        }

        private void CreateMp3CDTask_Click(object sender, EventArgs e)
        {
            RunProgram("Bwg.CreateDataDisk.exe", "-title \"Create MP3 Disc\" -ext mp3 -hide-image-settings");
        }

        private void CreateVideoVCD_Click(object sender, EventArgs e)
        {
            NotImplemented();
        }

        private void CreateVideoDVD_Click(object sender, EventArgs e)
        {
            NotImplemented();
        }

        private void CopyDiskButton_Click(object sender, EventArgs e)
        {
            RunProgram("Bwg.CopyDisk.exe");
        }

        private void GetInfo_Click(object sender, EventArgs e)
        {
            RunProgram("Bwg.GetInfo.exe");
        }

        private void DvdExplorer_Click(object sender, EventArgs e)
        {
            RunProgram("Bwg.DvdExplorer.exe");
        }

        private void ShowModules_Click(object sender, EventArgs e)
        {
            ModuleVersionForm form = new ModuleVersionForm(GetVersion());
            form.ShowDialog();
        }
        #endregion

        #region Help related methods
        private void HelpText_ButtonHelpRequested(object sender, HelpEventArgs args)
        {
            if (sender is TaskButton)
            {
                TaskButton b = (TaskButton)sender;
                Help.ShowHelp(this, GetHelpFilePath(), HelpNavigator.KeywordIndex, b.DescriptionText);
            }
            else
                Help.ShowHelp(this, GetHelpFilePath(), HelpNavigator.TableOfContents);
        }

        /// <summary>
        /// This is called when help is desired on a control in this form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        protected override void OnControlHelp(object sender, EventArgs args)
        {
            HelpText_ButtonHelpRequested(sender, null);
        }

        #endregion
    }
}