//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using Bwg.Scsi;

namespace Bwg.Burner
{
    /// <summary>
    /// This class represents a device IOCTL error
    /// </summary>
    public class DeviceIoctlError : DiskOperationError
    {
        /// <summary>
        /// The error code returned from windows
        /// </summary>
        public uint ErrorCode;

        /// <summary>
        /// The win32 function that was called
        /// </summary>
        public string CallName;

        /// <summary>
        /// Creates the IOCTL error object based on the win32 error code
        /// returned from the IOCTL call.
        /// </summary>
        /// <param name="code">the last error code returned by the win32 call</param>
        /// <param name="callname">the name of the WIN32 function called</param>
        public DeviceIoctlError(string callname, uint code)
        {
            // TODO - call FormatMessage() to get the string associated with this
            //        error message
            ErrorCode = code;
            CallName = callname;

            Message = "there was an error with a WIN32 call to function '" + CallName + "'" ;
            Message += " - LastError = " + code.ToString() ;
            Message += " - " + WinDev.Win32ErrorToString(code);
        }
    }
}
