//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;

namespace Bwg.Burner
{
    /// <summary>
    /// Disk operations for media that is not supported
    /// </summary>
    class NotSupportedDiskOperations : DiskOperations
    {
        #region constructor
        public NotSupportedDiskOperations(Drive d):base(d)
        {
        }
        #endregion

        #region public properties

        public override bool IsFormatSupported
        {
            get
            {
                return false;
            }
        }

        public override bool HasWriteProtectPassword
        {
            get
            {
                return false;
            }
        }

        public override bool IsWriteProtected
        {
            get
            {
                return false;
            }
        }

        public override bool IsWriteProtectSupported
        {
            get
            {
                return false;
            }
        }

        public override string WriteProtectPassword
        {
            set
            {
                throw new Exception("write protected password not supported");
            }
        }

        public override long CapacityInSectors
        {
            get
            {
                return 0;
            }
        }

        public override long WritableCapacityInSectors
        {
            get
            {
                return 0;
            }
        }

        public override long RemainingCapacityInWritableSectors
        {
            get
            {
                return 0;
            }
        }

        public override bool IsBurnSupported
        {
            get
            {
                return false;
            }
        }

        public override bool IsEraseSupported
        {
            get
            {
                return false;
            }
        }

        public override bool IsMultiSessionSupported
        {
            get
            {
                return false;
            }
        }

        public override bool IsSessionAtOnceSupported
        {
            get
            {
                return false;
            }
        }

        public override bool IsTrackAtOnceSupported
        {
            get
            {
                return false;
            }
        }
        #endregion

        #region public methods

        public override DiskOperationError BurnDisk(Drive.BurnType bt, DiskBurnImage d, bool multi_session, SpeedInfo speed, out ulong bytes)
        {
            bytes = 0;
            return NotSupported("BurnDisk");
        }

        public override DiskOperationError Erase(Drive.EraseType et)
        {
            return NotSupported("Erase");
        }

        #endregion
    }
}
