using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Bwg.Scsi ;

namespace Bwg.Burner
{
    class NullDataSource : DataSource
    {
        #region private member variables
        private string m_name;
        private long m_count;
        private int m_sector_size;
        #endregion

        public NullDataSource(string name, long count, int size)
        {
            m_name = name;
            m_count = count;
            m_sector_size = size;
        }

        public override string Name
        {
            get
            {
                return m_name;
            }
        }

        public override long SectorCount
        {
            get
            {
                return m_count;
            }
        }

        public override long SendData(Bwg.Scsi.WriteBufferPool pool, long start_sector, int sector_size, long lba)
        {
            long count = m_count;
            bool first = true;

            while (count > 0)
            {
                long todo = count;

                WriteBuffer buf = pool.GetFreeBuffer();
                if (buf.BufferSize < count * sector_size)
                    todo = buf.BufferSize / sector_size;

                //
                // Nothing here should exceed the size of a 32 bit integer.  However, we include
                // checks because any count of sectors is represented as a 64 bit number in anticipation
                // of larger and larger disks.
                //
                Debug.Assert(todo < Int32.MaxValue);

                buf.Zero();
                buf.DataSize = (int)todo;
                buf.SectorSize = sector_size;
                if (first)
                {
                    buf.LogicalBlockAddress = lba;
                    first = false;
                }
                else
                    buf.LogicalBlockAddress = long.MaxValue;

                count -= todo;
                pool.SendBufferToDevice(buf);
            }

            return m_count;
        }
    }
}
