//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using Bwg.Scsi ;

namespace Bwg.Burner
{
    /// <summary>
    /// This class represents a SCSI device error that has occurred
    /// </summary>
    public class ScsiDeviceError : DiskOperationError
    {
        /// <summary>
        /// The sense key retrieved from the device for this error
        /// </summary>
        public Device.SenseKeyType SenseKey;

        /// <summary>
        /// The ASC value retreived from the device for this error
        /// </summary>
        public byte ASC;

        /// <summary>
        /// The ASCQ value retreived from the device for this error
        /// </summary>
        public byte ASCQ;

        /// <summary>
        /// This method creates a new error object by querying the device for the appropriate
        /// values.
        /// </summary>
        /// <param name="dev">the device to query</param>
        public ScsiDeviceError(Device dev)
        {
            SenseKey = dev.GetSenseKey();
            ASC = dev.GetSenseAsc();
            ASCQ = dev.GetSenseAscq();

            Message = Device.LookupSenseError(ASC, ASCQ);
        }
    }
}
