//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using Bwg.Logging ;
using Bwg.Scsi ;
using Bwg.Burner ;

namespace Bwg.Burner
{
    /// <summary>
    /// This class represents a track that exists within a session on a disk
    /// </summary>
    public class Track
    {
        #region private member functions
        private HeaderData m_header;
        private SubheaderData m_subheader;
        #endregion

        #region public data members
        /// <summary>
        /// The track information associated with this track
        /// </summary>
        public readonly TrackInformation Info;
        #endregion

        #region public properties

        /// <summary>
        /// This property contains the header data for a sector in the track
        /// </summary>
        public HeaderData Header
        {
            get
            {
                return m_header;
            }
            set
            {
                m_header = value;
            }
        }

        /// <summary>
        /// THis property contains the subheader for the mode 2 data for this track
        /// </summary>
        public SubheaderData Subheader
        {
            get
            {
                return m_subheader;
            }
            set
            {
                m_subheader = value;
            }
        }

        /// <summary>
        /// Returns TRUE if this is an audio track
        /// </summary>
        public bool IsAudio
        {
            get
            {
                return (Info.TrackMode & 0x04) == 0;
            }
        }

        /// <summary>
        /// This method returns true if the data is mode 1 data.
        /// </summary>
        public bool IsDataMode1
        {
            get
            {
                if ((Info.TrackMode & 0x04) != 0)
                {
                    if (Info.DataMode == TrackInformation.DataModeType.Mode1)
                        return true;
                }
                return false;
            }
        }

        /// <summary>
        /// This property returns TRUE if the disk is a Data mode 2, form 1 disk
        /// </summary>
        public bool IsDataMode2
        {
            get
            {
                if ((Info.TrackMode & 0x04) != 0)
                {
                    // This really just detects mode 2 data
                    if (Info.DataMode == TrackInformation.DataModeType.Mode2)
                        return true;
                }
                return false ;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public bool IsDataMode2Formless
        {
            get
            {
                return IsDataMode2 && m_subheader == null;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public bool IsDataMode2Form1
        {
            get
            {
                return IsDataMode2 && m_subheader != null && (m_subheader.Submode & SubheaderData.SubmodeType.Form2) == 0;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public bool IsDataMode2Form2
        {
            get
            {
                return IsDataMode2 && m_subheader != null && (m_subheader.Submode & SubheaderData.SubmodeType.Form2) != 0;
            }
        }

        /// <summary>
        /// This property returns a human readable string describing the mode of the
        /// track.
        /// </summary>
        public string TrackModeString
        {
            get
            {
                string str = "" ;
                if ((Info.TrackMode & 0x04) != 0)
                {
                    str = "Data";
                    if (IsDataMode1)
                        str += " Mode 1";
                    else if (IsDataMode2Form1)
                        str += " Mode 2 Form 1";
                    else if (IsDataMode2Form2)
                        str += " Mode 2 Form 2";
                    else if (IsDataMode2Formless)
                        str += " Mode 2 Formless";
                    else
                        str += " Mode ?";
                }
                else
                {
                    str = "Audio";
                    if ((Info.TrackMode & 1) != 0)
                        str += " With PreEmphasis";
                }

                return str;
            }
        }
        #endregion

        #region constructors

        /// <summary>
        /// Create the track object
        /// </summary>
        /// <param name="info">the info about the track</param>
        public Track(TrackInformation info)
        {
            Info = info;
            m_header = null;
            m_subheader = null;
        }

        #endregion
    }
}
