using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Bwg.Burner;

namespace Bwg.CreateAudioDisk
{
    class CDTextDataBinaryConverter : TypeConverter
    {
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(CDTextDataBinary) || sourceType == typeof(string))
                return true;

            return base.CanConvertFrom(context, sourceType);
        }

        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(CDTextDataBinary))
                return true;

            return base.CanConvertTo(context, destinationType);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            if (value is CDTextDataBinary)
                return value.ToString();
            else if (value is string)
            {
                CDTextDataBinary binary;
                if (!ParseString((string)value, out binary))
                    throw new Exception("invalid format for binary field, expected a series of hex digits");

                return binary;
            }

            return base.ConvertFrom(context, culture, value);
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType == typeof(CDTextDataBinary) && value is string)
            {
                CDTextDataBinary binary;

                if (!ParseString((string)value, out binary))
                    throw new Exception("invalid format for binary field, expected a series of hex digits");

                return binary;
            }
            else if (destinationType == typeof(string) && value is CDTextDataGenre)
                return value.ToString();

            return base.ConvertTo(context, culture, value, destinationType);
        }

        public override bool IsValid(ITypeDescriptorContext context, object value)
        {
            CDTextDataBinary binary;

            if (value is string)
                return ParseString((string)value, out binary);

            return base.IsValid(context, value);
        }

        private byte Hex1ToByte(char ch)
        {
            byte b = 0;

            if (Char.IsDigit(ch))
            {
                b = (byte)(ch - '0') ;
            }
            else if (ch >= 'A' && ch <= 'F')
            {
                b = (byte)(ch - 'A' + 10) ;
            }
            else if (ch >= 'a' && ch <= 'f')
            {
                b = (byte)(ch - 'a' + 10) ;
            }
            else
            {
                throw new Exception("Illegal character for hex number '" + ch + "'") ;
            }

            return b;
        }

        private byte Hex2ToByte(char c1, char c2)
        {
            return (byte)(Hex1ToByte(c1) << 4 | Hex1ToByte(c2));
        }

        private bool ParseString(string str, out CDTextDataBinary binary)
        {
            binary = null;
            if (str == string.Empty)
                return true;

            StringBuilder b = new StringBuilder() ;
            foreach(char ch in str)
            {
                if (ch != ' ')
                    b.Append(ch) ;
            }
            str = b.ToString() ;

            int len = str.Length / 2;
            if ((str.Length & 1) != 0)
                len++;

            int i = 0;
            int j = 0;
            byte[] data = new byte[len];
            bool ret = true;

            try
            {

                if ((str.Length & 1) != 0)
                {
                    // Odd number of chars, take care of the first one first
                    data[j--] = Hex1ToByte(str[0]);
                    i++;
                }

                while (i < str.Length)
                {
                    data[j++] = Hex2ToByte(str[i], str[i + 1]);
                    i += 2;
                }
            }
            catch (Exception)
            {
                ret = false;
            }

            if (ret)
                binary = new CDTextDataBinary(data);
            return ret;
        }
    }
}
