using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.Globalization;
using Bwg.Burner;

namespace Bwg.CreateAudioDisk
{
    class CDTextDataGenreConverter : TypeConverter
    {
        public CDTextDataGenreConverter()
        {
        }

        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(CDTextDataGenre) || sourceType == typeof(string))
                return true;

            return base.CanConvertFrom(context, sourceType);
        }

        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(CDTextDataGenre))
                return true;

            return base.CanConvertTo(context, destinationType);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            if (value is CDTextDataGenre)
                return value.ToString();
            else if (value is string)
            {
                CDTextDataGenre genre;
                if (!ParseString((string)value, out genre))
                    throw new Exception("invalid format for genre field, expected 0xbbbb, \"optional string\"");

                return genre;
            }

            return base.ConvertFrom(context, culture, value);
        }

        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType == typeof(CDTextDataGenre) && value is string)
            {
                CDTextDataGenre genre;
                if (!ParseString((string)value, out genre))
                    throw new Exception("invalid format for genre field, expected 0xbbbb, \"optional string\"");
            }

            else if (destinationType == typeof(string) && value is CDTextDataGenre)
                return value.ToString();

            return base.ConvertTo(context, culture, value, destinationType);
        }

        public override bool IsValid(ITypeDescriptorContext context, object value)
        {
            CDTextDataGenre genre ;

            if (value is string)
                return ParseString((string)value, out genre);

            return base.IsValid(context, value);
        }

        private bool ParseString(string str, out CDTextDataGenre genre)
        {
            genre = null ;
            if (str == string.Empty)
                return true;

            string white = "[ \t]*";
            string pfxhx = "0x([0-9a-fA-F]+)" ;
            string hx = "([0-9a-fA-F]+)" ;
            string quoted_string = "\"(.*)\"";
            string unquoted_string = "(.*)";

            string hxnum ;
            string userstr ;

            Regex reg1 = new Regex("^" + pfxhx + white + "," + white + quoted_string + "$");
            Regex reg2 = new Regex("^" + hx + white + "," + white + quoted_string + "$");
            Regex reg3 = new Regex("^" + pfxhx + white + "$");
            Regex reg4 = new Regex("^" + hx + white + "$");
            Regex reg5 = new Regex("^" + pfxhx + white + "," + white + unquoted_string + "$");
            Regex reg6 = new Regex("^" + hx + white + "," + white + unquoted_string + "$");

            if (reg1.IsMatch(str))
            {
                Match m = reg1.Match(str);
                hxnum = m.Groups[1].Value;
                userstr = m.Groups[2].Value;
            }
            else if (reg2.IsMatch(str))
            {
                Match m = reg2.Match(str);
                hxnum = m.Groups[1].Value;
                userstr = m.Groups[2].Value;
            }
            else if (reg3.IsMatch(str))
            {
                Match m = reg3.Match(str);
                hxnum = m.Groups[1].Value;
                userstr = string.Empty;
            }
            else if (reg4.IsMatch(str))
            {
                Match m = reg4.Match(str);
                hxnum = m.Groups[1].Value;
                userstr = string.Empty;
            }
            else if (reg5.IsMatch(str))
            {
                Match m = reg5.Match(str);
                hxnum = m.Groups[1].Value;
                userstr = m.Groups[2].Value;
            }
            else if (reg6.IsMatch(str))
            {
                Match m = reg6.Match(str);
                hxnum = m.Groups[1].Value;
                userstr = m.Groups[2].Value;
            }
            else
                return false;

            int value;
            if (!Int32.TryParse(hxnum, NumberStyles.HexNumber, null, out value))
                return false ;

            byte [] data = new byte[2] ;
            data[1] = (byte)(value & 0xff) ;
            data[0] = (byte)((value >> 8) & 0xff) ;

            genre = new CDTextDataGenre(data, userstr, null) ;
            return true;
        }
    }
}
