//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;

namespace Bwg.Database.Freedb
{
    class FreeDBTransportHttp : FreeDBTransport
    {
        #region private member vars
        /// <summary>
        /// The host name of the HTTP host serving CDDB information.  This string includes all of the information
        /// required to contact the host (e.g. http://us.freedb.org:80/~cddb/cddb.cgi)
        /// </summary>
        private string m_host;

        /// <summary>
        /// This is all of the information about the user and his/her application
        /// </summary>
        private FreeDBUserInfo m_user_info;

        /// <summary>
        /// The protocol to use
        /// </summary>
        private int m_protocol;
        #endregion

        #region constructor
        public FreeDBTransportHttp(FreeDBUserInfo info, string host)
        {
            m_user_info = info;
            m_host = host;
            m_protocol = 1;
        }
        #endregion

        #region public methods

        public override bool SetProtocol(int p)
        {
            m_protocol = p;
            return true;
        }

        public override bool Start()
        {
            return true;
        }

        public override bool Finish()
        {
            return true;
        }

        public override RequestStatus SendCommandGetResponse(string cmd, out int code, out object response)
        {
            code = -1 ;
            response = null ;

            string str = m_host + "?" + CreateCmd(cmd);
            WebRequest webreq = WebRequest.Create(str);
            webreq.Credentials = CredentialCache.DefaultCredentials;
            HttpWebResponse resp ;

            try
            {
                resp = (HttpWebResponse)webreq.GetResponse();
            }
            catch (Exception)
            {
                return RequestStatus.CannotConnect;
            }

            Stream dataStream = resp.GetResponseStream();
            StreamReader reader = new StreamReader(dataStream);
            string respstr = reader.ReadToEnd();

            string[] seps = new string[] { "\r\n" };
            string [] lines = respstr.Split(seps, StringSplitOptions.RemoveEmptyEntries) ;

            // Line 0, should contains the response code
            code = GetResponseCode(ref lines[0]) ;
            if (code == -1)
                return RequestStatus.InvalidReponse;

            if (lines[0].Contains("until terminating"))
            {
                // Parse a multiline response
                string [] strarr = new string[lines.GetLength(0) - 2] ;
                for(int i = 1 ; i < lines.GetLength(0) - 1 ; i++)
                    strarr[i - 1] = lines[i] ;
                response = strarr ;
            }
            else
            {
                // Parse a single line response
                response = lines[0] ;
            }

            return RequestStatus.Success;
        }

        public override void Dispose()
        {
        }

        #endregion

        #region private methods
        private string CreateCmd(string cmd)
        {
            string str;

            str = "cmd=" + cmd.Replace(' ', '+');
            str += "&hello=" + m_user_info.UserName + "+" + m_user_info.UserHost + "+" + m_user_info.Client + "+" + m_user_info.ClientVersion;
            str += "&proto=" + m_protocol.ToString();

            return str;
        }
        #endregion
    }
}
