//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Bwg.Burner;
using Bwg.Scsi;

namespace Bwg.DiskImage
{
    class TrackInfoDataSource : DataSource
    {
        #region private member variables
        private DiskContents m_contents;
        private TrackInfo m_info;
        private int m_total;
        #endregion

        #region constructors
        public TrackInfoDataSource(DiskContents dc, TrackInfo ti, int total)
        {
            m_contents = dc;
            m_info = ti;
            m_total = total;
        }
        #endregion

        #region public methods
        public override string Name
        {
            get
            {
                return m_info.Number.ToString() + " of " + m_total.ToString();
            }
        }

        public override long SectorCount
        {
            get
            {
                return m_info.Size;
            }
        }

        public override long SendData(Bwg.Scsi.WriteBufferPool pool, long start_sector, int sector_size, long lba)
        {
            int tocopy = m_info.Size ;
            int current = m_info.Start ;
            int blocks = pool.PageSize / sector_size;

            Debug.Assert(sector_size == m_info.SectorSize);

            Debug.WriteLine("TrackInfoDataSource.SendData() - sending " + tocopy.ToString() + " sectors of data");
            while (tocopy > 0)
            {
                int toread = tocopy;
                if (toread > blocks)
                    toread = blocks;

                WriteBuffer buf = pool.GetFreeBuffer() ;
                DiskOperationError status = m_contents.ReadTrackData(current, toread, sector_size, buf.BufferPtr) ;
                if (status != null)
                {
                    Debug.WriteLine("ReadTrackData failed - " + status.Message);
                    throw new Exception(status.Message);
                }

                buf.SectorSize = sector_size;
                buf.DataSize = toread;
                pool.SendBufferToDevice(buf);

                tocopy -= toread;
                current += toread;
            }

            return m_info.Size;
        }
        #endregion
    }
}
