using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.DvdMaster
{
    /// <summary>
    /// This class represents a single cell of video.  A cell is identified by both the
    /// cell id (byte) and the vob id (word)
    /// </summary>
    public class DvdCell
    {
        //
        // TODO:
        // o  Mask and highlight layer
        // o  Start and End sector on disk

        #region private member variables

        /// <summary>
        /// The program chain that this cell belongs to.
        /// </summary>
        private DvdProgramChain m_chain;

        /// <summary>
        /// This is the number of angles for the cell
        /// </summary>
        private int m_number_angles;

        /// <summary>
        /// The CELL ID of this cell
        /// </summary>
        private byte m_cell_id;

        /// <summary>
        /// The vob ID of this cell
        /// </summary>
        private ushort m_vob_id;

        /// <summary>
        /// The cell command associated with this cell (or null if none)
        /// </summary>
        private DvdCommandList m_cell_command;
        #endregion

        #region constructor
        /// <summary>
        /// This creates a new cell given a cell id and the vob id
        /// </summary>
        /// <param name="cid">the cell id</param>
        /// <param name="vid">the vob id</param>
        public DvdCell(DvdProgramChain chain, byte cid, ushort vid)
        {
            m_chain = chain;
            m_number_angles = 1;
            m_cell_id = cid;
            m_vob_id = vid;
            m_cell_command = null;
        }

        /// <summary>
        /// This creates a new cell given a cell id and the vob id and the number of angles
        /// </summary>
        /// <param name="cid">the cell id</param>
        /// <param name="vid">the vob id</param>
        public DvdCell(DvdProgramChain chain, byte cid, ushort vid, int angles)
        {
            m_chain = chain;
            m_number_angles = angles;
            m_cell_id = cid;
            m_vob_id = vid;
            m_cell_command = null;
        }

        /// <summary>
        /// This method creates and invalid cell with no cell id or
        /// vob id.
        /// </summary>
        public DvdCell(DvdProgramChain chain)
        {
            m_chain = chain;
            m_cell_id = 0;
            m_vob_id = 0;
        }
        #endregion

        #region public properties
        /// <summary>
        /// This method returns true if the cell is valid, otherwise it returns false.
        /// </summary>
        public bool IsValid
        {
            get { return m_cell_id != 0 && m_vob_id != 0; }
        }

        /// <summary>
        /// This method returns or sets the cell id of the cell
        /// </summary>
        public byte CellId
        {
            get { return m_cell_id; }
            set { m_cell_id = value; }
        }

        /// <summary>
        /// This method returns or sets the vob id of the cell.
        /// </summary>
        public ushort VobId
        {
            get { return m_vob_id; }
            set { m_vob_id = value; }
        }

        /// <summary>
        /// This property returns or sets the cell commands assocaited with this cell
        /// </summary>
        public DvdCommandList CellCommand
        {
            get { return m_cell_command; }
            set { Debug.Assert(value == null || value.Count <= 8);  m_cell_command = value; }
        }
        #endregion
    }
}
