using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.DvdMaster
{
    /// <summary>
    /// This class represents a DVD program.  A program is a group of cells within
    /// a given DvdProgramChain.
    /// </summary>
    public class DvdProgram
    {
        #region private member variables
        /// <summary>
        /// The program chain that contains the cells that make up this DvdProgram
        /// </summary>
        private DvdProgramChain m_chain;

        /// <summary>
        /// The cell number for the starting cell of this program
        /// </summary>
        private byte m_start_cell;

        /// <summary>
        /// The cell number for the ending cell of this program
        /// </summary>
        private byte m_end_cell;
        #endregion

        #region constructor
        /// <summary>
        /// This constructor creates a program with valid start and end cell values
        /// </summary>
        /// <param name="start"></param>
        /// <param name="end"></param>
        public DvdProgram(DvdProgramChain chain, byte start, byte end)
        {
            m_start_cell = start;
            m_end_cell = end;
        }

        public DvdProgram(DvdProgramChain chain)
        {
            m_chain = chain;
            m_start_cell = 0;
            m_end_cell = 0;
        }
        #endregion

        #region public properties
        /// <summary>
        /// This property returns true if the program is valid
        /// </summary>
        public bool IsValid
        {
            get { return m_start_cell != 0 && m_end_cell != 0; }
        }

        public byte StartCell
        {
            get { return m_start_cell; }
            set { Debug.Assert(m_chain.IsValidCell(value)) ; m_start_cell = value; }
        }

        public byte EndCell
        {
            get { return m_end_cell; }
            set { Debug.Assert(m_chain.IsValidCell(value)) ; m_end_cell = value; }
        }
        #endregion
    }
}
