using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics ;

namespace Bwg.DvdMaster
{
    /// <summary>
    /// A program chain is a group of one cells.
    /// </summary>
    public class DvdProgramChain
    {
        #region private member variables
        /// <summary>
        /// The list of cells that make up this program chain
        /// </summary>
        private IList<DvdCell> m_cells;

        /// <summary>
        /// The set of commands to execute before this program chain is played
        /// </summary>
        private DvdCommandList m_pre_commands;

        /// <summary>
        /// The set of commands to execute after this program chain is played
        /// </summary>
        private DvdCommandList m_post_commands;
        #endregion

        #region constructor
        /// <summary>
        /// This constructor creates an empty program chain.
        /// </summary>
        public DvdProgramChain()
        {
            m_cells = new List<DvdCell>();
        }
        #endregion

        #region public properties
        /// <summary>
        /// This property returns the number of cells in this program chain
        /// </summary>
        public int Count
        {
            get { return m_cells.Count; }
        }

        /// <summary>
        /// This property is the command list that is executed before the program chain is displayed
        /// </summary>
        public DvdCommandList PreCommands
        {
            get { return m_pre_commands; }
            set { Debug.Assert(value.Count <= 128); m_pre_commands = value; }
        }
        
        /// <summary>
        /// This property is the command list that is executed after the program chain is displayed
        /// </summary>
        public DvdCommandList PostCommands
        {
            get { return m_post_commands; }
            set { Debug.Assert(value.Count <= 128); m_post_commands = value; }
        }
        #endregion

        #region public methods
        /// <summary>
        /// This method returns true if the cell id is of a valid cell in the
        /// program chain.
        /// </summary>
        /// <param name="cellid">the cell id of the cell we are looking for</param>
        /// <returns>true if the id is valid, false otherwise</returns>
        public bool IsValidCell(byte cellid)
        {
            foreach (DvdCell c in m_cells)
            {
                if (c.IsValid && c.CellId == cellid)
                    return true;
            }
            return false;
        }
        #endregion
    }
}
