//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading ;
using System.Diagnostics;
using Bwg.Logging;
using Bwg.Scsi;
using Bwg.Burner;
using Bwg.Controls;
using Bwg.Hardware ;

namespace Bwg.GetInfo
{
    public partial class GetInfoForm : BwgBurnBaseForm
    {
        private Drive m_drive;

        public GetInfoForm(string [] args)
        {
            InitializeComponent();

            LoggerWindow.Sink = m_sink;
            LoggerWindow.SetupColumns();

            m_logger.LogMessage(new UserMessage(UserMessage.Category.Info, 0, "Get Disk Information Version " + GetVersion()));

            DeviceSelector.Logger = m_logger;
            DeviceSelector.SelectedIndexChanged += new EventHandler(DeviceSelector_SelectedIndexChanged);
            DeviceSelector.InitializeDeviceList(false);

            if (args.GetLength(0) > 0)
            {
                CloseSession.Visible = true;
                CloseTrack.Visible = true;
            }
            else
            {
                CloseSession.Visible = false;
                CloseTrack.Visible = false;
            }
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);
        }

        private void UpdateWindows(Disk dsk)
        {
            SpeedText.Text = "";
            FeatureList.Items.Clear();

            PropertyGrid.SelectedObject = null;
            DiskContents.Nodes.Clear();

            Profile.Text = "No Media Loaded";
            if (m_drive == null)
                return;

            Profile.Text = m_drive.StatusString;

            UpdateSpeedTab();
            UpdateFeatureTab();
            UpdateContentTab(dsk);
            UpdateHardwareTab();
            UpdateDLInfoTab();
            UpdateDataTab();
        }

        private void UpdateCDText()
        {
        }

        private int DataRateToSpeed(int rate)
        {
            int ret = 0 ;

            if (m_drive.IsCd)
                ret = rate / 176400;
            else
                ret = rate / 1385000;

            return ret;
        }

        private void UpdateSpeedTab()
        {
            Device.CommandStatus st;
            int mult;
            PerformanceList plist;

            SpeedText.Text = string.Empty;

            st = m_drive.Device.SetCdSpeed(Device.RotationalControl.CLVandNonPureCav, Device.OptimumSpeed, Device.OptimumSpeed);
            if (st != Device.CommandStatus.Success)
                SpeedText.Text += "SetCdSpeed() failed\r\n\r\n";

            SpeedText.Text += "Read Performance Data:\r\n";
            if (m_drive.DriveModel.Contains("DVD RW DRU-810A"))
                SpeedText.Text += "    This sony drive does not do the SCSI GET PERFORMANCE request\r\n";
            else
            {
                st = m_drive.Device.GetPerformance(0, PerformanceList.DataType.ReadData, PerformanceList.ExceptType.Entire, out plist);
                if (st != Device.CommandStatus.Success)
                {
                    string str = "    Error acquiring read speed performance data - ";
                    if (st == Device.CommandStatus.DeviceFailed)
                        str += Device.LookupSenseError(m_drive.Device.GetSenseAsc(), m_drive.Device.GetSenseAscq());
                    else
                        str += "IOCTL_SCSI_PASS_THROUGH_DIRECT failed, error code = " + m_drive.Device.LastError.ToString();

                    str += "\r\n";
                    SpeedText.Text += str;
                }
                else
                {
                    foreach (Performance p in plist)
                    {
                        mult = DataRateToSpeed((int)p.StartPerformance * 1000);
                        SpeedText.Text += mult.ToString() + "X: ";
                        SpeedText.Text += "    " + p.ToString() + "\r\n";
                    }
                }
            }
            SpeedText.Text += "\r\n";

            SpeedText.Text += "Write Performance Data:\r\n";
            if (m_drive.DriveModel.Contains("DVD RW DRU-810A"))
                SpeedText.Text += "    This sony drive does not do the SCSI GET PERFORMANCE request\r\n";
            else
            {
                st = m_drive.Device.GetPerformance(0, PerformanceList.DataType.WriteData, PerformanceList.ExceptType.Entire, out plist);
                if (st != Device.CommandStatus.Success)
                {
                    string str = "    Error acquiring write speed performance data - ";
                    if (st == Device.CommandStatus.DeviceFailed)
                        str += Device.LookupSenseError(m_drive.Device.GetSenseAsc(), m_drive.Device.GetSenseAscq());
                    else
                        str += "IOCTL_SCSI_PASS_THROUGH_DIRECT failed, error code = " + m_drive.Device.LastError.ToString();
                    str += "\r\n";
                    SpeedText.Text += str;
                }
                else
                {
                    foreach (Performance p in plist)
                    {
                        mult = DataRateToSpeed((int)p.StartPerformance * 1000);
                        SpeedText.Text += mult.ToString() + "X: ";
                        SpeedText.Text += "    " + p.ToString() + "\r\n";
                    }
                }
            }
            SpeedText.Text += "\r\n";

            SpeedText.Text += "Write Speed Descriptors:\r\n";
            SpeedDescriptorList slist;
            st = m_drive.Device.GetSpeed(out slist);
            if (st != Device.CommandStatus.Success)
            {
                string str = "    Error acquiring write speed descriptors - ";
                if (st == Device.CommandStatus.DeviceFailed)
                    str += Device.LookupSenseError(m_drive.Device.GetSenseAsc(), m_drive.Device.GetSenseAscq());
                else
                    str += "IOCTL_SCSI_PASS_THROUGH_DIRECT failed, error code = " + m_drive.Device.LastError.ToString();
                str += "\r\n";
                SpeedText.Text += str;
            }
            else
            {
                foreach (SpeedDescriptor s in slist)
                {
                    mult = DataRateToSpeed(s.ReadSpeed * 1024);
                    SpeedText.Text += mult.ToString() + "X/";
                    mult = DataRateToSpeed(s.WriteSpeed * 1024);
                    SpeedText.Text += mult.ToString() + "X: ";
                    SpeedText.Text += "    " + s.ToString() + "\r\n";
                }
            }
            SpeedText.Text += "\r\n";

            SpeedText.Text += "Mode Page 2A speed descriptors\r\n";
            ModeTable table;
            st = m_drive.Device.ModeSense(Device.PageControl.Current, 0x2A, out table);
            if (st != Device.CommandStatus.Success)
            {
                string str = "    Error reading mode page 2A - ";
                if (st == Device.CommandStatus.DeviceFailed)
                    str += Device.LookupSenseError(m_drive.Device.GetSenseAsc(), m_drive.Device.GetSenseAscq());
                else
                    str += "IOCTL_SCSI_PASS_THROUGH_DIRECT failed, error code = " + m_drive.Device.LastError.ToString();
                str += "\r\n";
            }
            else
            {
                ModePage pg = table.Pages[0];
                if (pg.Length >= 30)
                {
                    SpeedText.Text += "    Current Write Speed: " + pg.ModeGet16(28) + "\r\n";
                    if (pg.Length >= 32)
                    {
                        int len = pg.ModeGet16(30);
                        for (int i = 0; i < len; i++)
                        {
                            int offset = 32 + i * 4 ;
                            SpeedText.Text += "    Write Speed: " + pg.ModeGet16(offset + 2).ToString() + "\r\n";
                        }
                    }
                }
            }
        }

        private void UpdateFeatureTab()
        {
            DiskOperationError status;

            FeatureList flist;
            status = m_drive.GetFeatureList(Device.GetConfigType.AllFeatures, out flist);
            if (status != null)
            {
                string str = "error reading feature list from device - " + status.Message;
                FeatureList.Items.Add(str);
            }
            else
            {
                foreach (Feature f in flist.Features)
                {
                    ListViewItem item = new ListViewItem(f.Name);

                    if (f.Code == Feature.FeatureType.ProfileList)
                    {
                        bool bfirst = true;
                        foreach (Drive.SupportedProfiles p in m_drive.ProfileList)
                        {
                            if (!bfirst)
                                item = new ListViewItem("");

                            item.SubItems.Add("");
                            item.SubItems.Add("");
                            item.SubItems.Add(p.ToString()) ;
                            bfirst = false;
                            FeatureList.Items.Add(item);
                        }
                    }
                    else if (f.Code == Feature.FeatureType.LogicalUnitSerialNumber)
                    {
                        string str = Encoding.ASCII.GetString(f.Data);
                        item.SubItems.Add(f.Version.ToString());
                        item.SubItems.Add(f.Current ? "Yes" : "No");
                        item.SubItems.Add("'" + str + "'");
                        FeatureList.Items.Add(item);
                    }
                    else
                    {
                        item.SubItems.Add(f.Version.ToString());
                        item.SubItems.Add(f.Current ? "Yes" : "No");

                        if (f.Current)
                        {
                            string str = string.Empty;
                            int i = 0;
                            int thisline = 0;
                            while (i < f.Data.GetLength(0))
                            {
                                if (item == null)
                                {
                                    item = new ListViewItem("");
                                    item.SubItems.Add("");
                                    item.SubItems.Add("");
                                }
                                str += " " + f.Data[i].ToString("X2");
                                thisline++;
                                i++;

                                if (thisline == 16)
                                {
                                    item.SubItems.Add(str);
                                    FeatureList.Items.Add(item);
                                    item = null;
                                    str = string.Empty;
                                    thisline = 0;
                                }
                            }

                            if (item != null)
                            {
                                item.SubItems.Add(str);
                                FeatureList.Items.Add(item);
                            }
                        }
                        else
                        {
                            item.SubItems.Add("");
                            FeatureList.Items.Add(item);
                        }
                    }
                }
            }
        }

        private void UpdateContentTab(Disk dsk)
        {
            Device.CommandStatus st;
            DiscInformation info;

            st = m_drive.Device.ReadDiscInformation(out info);
            if (st != Device.CommandStatus.Success)
            {
                if (m_drive.Device.GetSenseAsc() == 0x3a && (m_drive.Device.GetSenseAscq() == 0 || m_drive.Device.GetSenseAscq() == 1))
                {
                    DiskContents.Nodes.Add("No Media Loaded");
                }
                else
                {
                    string str = "Error reading information from the disk - " + Device.LookupSenseError(m_drive.Device.GetSenseAsc(), m_drive.Device.GetSenseAscq());
                    MessageBox.Show(str, "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                return;
            }

            PropertyGrid.SelectedObject = info;
            UpdateContentsTree(dsk);

            DiskContents.SelectedNode = DiskContents.Nodes[0];
        }

        private void UpdateContentsTree(Disk dsk)
        {
            if (!dsk.IsValid)
            {
                DiskContents.Nodes.Add("Error reading disk");
                return;
            }

            string main = dsk.MediaType + " - ";
            if (dsk.Sessions.GetLength(0) > 1)
                main += "Multi Session Disk";
            else
                main += "Single Session Disk";

            if (dsk.IsEmpty)
                main += ", empty disk";
            else if (dsk.LeadOutStart != null)
                main += ", leadout " + dsk.LeadOutStart.ToString();

            TreeNode node = new TreeNode(main);
            DiskContents.Nodes.Add(node);

            if (dsk.CDText != null && !dsk.CDText.IsEmpty)
                AddCDTextToTree(node, 0, dsk.CDText);

            AddSessionsToTree(node, dsk);

            DiskContents.ExpandAll();
        }

        private void AddCDTextToTree(TreeNode node, int trackno, CDTextEncoderDecoder cdtext)
        {
            TreeNode cdnode = new TreeNode("CD Text");
            node.Nodes.Add(cdnode);

            for (byte i = 0; i < cdtext.LanguageCount ; i++)
            {
                CDTextEncoderDecoder.LanguageType lang = cdtext.GetLanguage(i);
                TreeNode langnode = new TreeNode(lang.ToString());
                cdnode.Nodes.Add(langnode);

                IList<CDTextField> list = cdtext.GetCDTextList(trackno);
                if (list != null)
                {
                    foreach (CDTextField field in list)
                    {
                        if (field.LanguageIndex == i)
                        {
                            string str;

                            str = field.Type.ToString() + " = " + field.Value.ToString();
                            TreeNode item = new TreeNode(str);
                            langnode.Nodes.Add(item);
                        }
                    }
                }
            }
        }

        private void AddSessionsToTree(TreeNode parent, Disk dsk)
        {
            for (int i = 0; i < dsk.Sessions.GetLength(0); i++)
            {
                Session s = dsk.Sessions[i];
                TreeNode node = new TreeNode("Session " + (i + 1).ToString());

                parent.Nodes.Add(node);
                AddSessionToNode(node, dsk, s);
            }
        }

        private void AddSessionToNode(TreeNode node, Disk dsk, Session s)
        {
            for (int i = 0; i < s.Tracks.GetLength(0); i++)
            {
                Track t = s.Tracks[i];

                string str = "Track - " + t.Info.TrackNumber.ToString();
                str += "  " + t.TrackModeString;
                MinuteSecondFrame msf = new MinuteSecondFrame(t.Info.TrackStartAddress);
                str += ", Start = " + msf.ToString();
                msf = new MinuteSecondFrame(t.Info.TrackSize);
                str += ", Length = " + msf.ToString();

                TreeNode n = new TreeNode(str);
                n.Tag = t;
                node.Nodes.Add(n);

                if (dsk.CDText != null && !dsk.CDText.IsEmpty)
                    AddCDTextToTree(n, i + 1, dsk.CDText);
            }
        }

        private void DeviceSelector_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Close the drive that is currently open
            if (m_drive != null)
                m_drive.Dispose();

            m_drive = null;
            string dname = (string)DeviceSelector.SelectedItem;
            Device dev = new Device(m_logger);
            if (!dev.Open(dname[0]))
            {
                MessageBox.Show("Cannot open device '" + dname + "'", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                m_drive = new Drive(dev);
                DiskOperationError status = m_drive.Initialize();
                if (status != null)
                {
                    m_drive = null;
                    MessageBox.Show("Cannot initialize drive '" + dname + "' - " + status.Message, "Error!",
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            if (m_drive != null)
            {
                Disk dsk = new Disk(m_drive);
                UpdateWindows(dsk);
            }

            if (m_drive != null)
            {
                m_drive.Dispose();
                m_drive = null;
            }
        }

        private void Unlock_Click(object sender, EventArgs e)
        {
            Debug.Assert(m_drive == null);

            string dname = (string)DeviceSelector.SelectedItem;
            Device dev = new Device(m_logger);
            if (!dev.Open(dname[0]))
            {
                MessageBox.Show("Cannot open device '" + dname + "'", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                m_drive = new Drive(dev);
                DiskOperationError status = m_drive.Initialize();
                if (status != null)
                {
                    m_drive = null;
                    MessageBox.Show("Cannot initialize drive '" + dname + "' - " + status.Message, "Error!",
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            m_drive.Device.PreventAllowMediumRemoval(Device.PreventAllow.Allow);

            m_drive.Dispose();
            m_drive = null;
        }

        private void Eject_Click(object sender, EventArgs e)
        {
            Debug.Assert(m_drive == null);
            string dname = (string)DeviceSelector.SelectedItem;
            Device dev = new Device(m_logger);
            if (!dev.Open(dname[0]))
            {
                MessageBox.Show("Cannot open device '" + dname + "'", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                m_drive = new Drive(dev);
                DiskOperationError status = m_drive.Initialize();
                if (status != null)
                {
                    m_drive = null;
                    MessageBox.Show("Cannot initialize drive '" + dname + "' - " + status.Message, "Error!",
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            m_drive.Device.StartStopUnit(false, Device.PowerControl.NoChange, Device.StartState.EjectDisc);

            m_drive.Dispose();
            m_drive = null;
        }

        private void DiskContents_AfterSelect(object sender, TreeViewEventArgs e)
        {
            bool enable = false;

            if (e.Node.Tag != null && e.Node.Tag is Track)
            {
                Track t = (Track)e.Node.Tag;
                if (t.IsDataMode1 || t.IsDataMode2Form1)
                    enable = true;
            }
            Extract.Visible = enable;
            Extract.Enabled = enable;
        }

        private void ProgressChanged(object sender, ProgressChangedEventArgs args)
        {
            if (InvokeRequired)
            {
                object[] param = new object[2];
                param[0] = sender;
                param[1] = args;
                Invoke(new ProgressChangedEventHandler(ProgressChanged), param);
            }
            else
            {
                if (args.UserState is ProgressText)
                {
                    ProgressText pt = (ProgressText)args.UserState;
                    ProgressBar.Text = pt.LongName;
                }
                else if (args.UserState is string)
                {
                    ProgressBar.Text = (string)args.UserState;
                }
                else
                {
                    ProgressBar.Text = "????";
                }

                ProgressBar.Value = args.ProgressPercentage;
                Application.DoEvents();
            }
        }

        private void Extract_Click(object sender, EventArgs e)
        {
            TreeNode node = DiskContents.SelectedNode;
            if (node == null)
                return;

            if (node.Tag == null || !(node.Tag is Track))
                return;

            DiskOperationError status;
            Debug.Assert(m_drive == null);
            string dname = (string)DeviceSelector.SelectedItem;
            Device dev = new Device(m_logger);
            if (!dev.Open(dname[0]))
            {
                MessageBox.Show("Cannot open device '" + dname + "'", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                m_drive = new Drive(dev);
                status = m_drive.Initialize();
                if (status != null)
                {
                    m_drive = null;
                    MessageBox.Show("Cannot initialize drive '" + dname + "' - " + status.Message, "Error!",
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            Track t = (Track)node.Tag;

            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = "ISO Image Files (*.iso)|*.iso|All files (*.*)|*.*";
            dialog.RestoreDirectory = true;
            dialog.AddExtension = true;
            dialog.CheckPathExists = true;
            dialog.DefaultExt = "iso";
            dialog.FilterIndex = 0;

            DialogResult res = dialog.ShowDialog();
            if (res == DialogResult.OK)
            {
                bool success = true;
                m_drive.ProgressChanged += new ProgressChangedEventHandler(ProgressChanged);
                Eject.Enabled = false;
                Unlock.Enabled = false;
                Extract.Enabled = false;
                try
                {
                    m_drive.WriteTrackToFile(t, dialog.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("cannot write file '" + dialog.FileName + "' - " + ex.Message, "Error!");
                    success = false;
                }
                m_drive.ProgressChanged -= new ProgressChangedEventHandler(ProgressChanged);
                Eject.Enabled = true;
                Unlock.Enabled = true;
                Extract.Enabled = true;

                ProgressBar.Value = 100;
                if (success)
                    MessageBox.Show("Track written to file '" + dialog.FileName + "' sucessfully", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);

                ProgressBar.Value = 0;
                ProgressBar.Text = "GetInfo";
            }

            m_drive.Dispose();
            m_drive = null;
        }

        private void CloseTrack_Click(object sender, EventArgs e)
        {
            DiskOperationError status;

            Debug.Assert(m_drive == null);
            string dname = (string)DeviceSelector.SelectedItem;
            Device dev = new Device(m_logger);
            if (!dev.Open(dname[0]))
            {
                MessageBox.Show("Cannot open device '" + dname + "'", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                m_drive = new Drive(dev);
                status = m_drive.Initialize();
                if (status != null)
                {
                    m_drive = null;
                    MessageBox.Show("Cannot initialize drive '" + dname + "' - " + status.Message, "Error!",
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            DiscInformation info ;
            Device.CommandStatus st ;

            st = dev.ReadDiscInformation(out info) ;
            if (st != Device.CommandStatus.Success)
            {
                MessageBox.Show("Error reading disc information");
                return;
            }

            status = m_drive.CloseTrackSession(Device.CloseTrackSessionType.CloseTrack, info.LastTrackInLastSession);
            if (status != null)
                MessageBox.Show(status.Message);
            else
                MessageBox.Show("Success");

            m_drive.Dispose();
            m_drive = null;
        }

        private void CloseSession_Click(object sender, EventArgs e)
        {
            DiskOperationError status;
            Debug.Assert(m_drive == null);
            string dname = (string)DeviceSelector.SelectedItem;
            Device dev = new Device(m_logger);
            if (!dev.Open(dname[0]))
            {
                MessageBox.Show("Cannot open device '" + dname + "'", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                m_drive = new Drive(dev);
                status = m_drive.Initialize();
                if (status != null)
                {
                    m_drive = null;
                    MessageBox.Show("Cannot initialize drive '" + dname + "' - " + status.Message, "Error!",
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }


            status = m_drive.CloseTrackSession(Device.CloseTrackSessionType.CloseSession, 0);
            if (status != null)
                MessageBox.Show(status.Message);
            else
                MessageBox.Show("Success");

            m_drive.Dispose();
            m_drive = null;
        }

        private void UpdateDLInfoTab()
        {
            DLInfoList.Items.Clear();

            if (m_drive.IsDualLayer)
            {
                uint value ;
                bool isfixed ;

                Device.CommandStatus st ;

                DLTab.Text = "DL Info";
                if (!MainTabControl.TabPages.Contains(DLTab))
                {
                    int index = MainTabControl.TabPages.IndexOf(Hardware);
                    MainTabControl.TabPages.Insert(index + 1, DLTab);
                }

                ListViewItem item;

                st = m_drive.Device.ReadDvdLayer0Size(out isfixed, out value);
                item = new ListViewItem("Layer 0 Boundary Fixed");
                if (st == Device.CommandStatus.Success)
                    item.SubItems.Add(isfixed.ToString());
                else
                    item.SubItems.Add("error reading info");
                item.SubItems.Add("If true, the layer 0 data zone size if fixed, otherwise it has yet to be fixed.");
                DLInfoList.Items.Add(item);

                item = new ListViewItem("Layer 0 Boundary Size");
                if (st == Device.CommandStatus.Success)
                    item.SubItems.Add(value.ToString());
                else
                    item.SubItems.Add("error reading info");
                item.SubItems.Add("The size of the layer 0 data zone, default value from ADIP if not fixed.");
                DLInfoList.Items.Add(item);
            }
            else if (m_drive.IsBluRay)
            {
                DLTab.Text = "Blu-Ray Info";
            }
            else
            {
                if (MainTabControl.TabPages.Contains(DLTab))
                    MainTabControl.TabPages.Remove(DLTab);
            }
        }

        private void UpdateHardwareTab()
        {
            HardwareManager mgr = new HardwareManager();

            HardwareList.Items.Clear();
            if (mgr.OpenMachine(string.Empty))
            {
                foreach (HardwareDevice dev in mgr)
                {
                    ListViewItem item = new ListViewItem(dev.Name);
                    item.SubItems.Add(dev.Class);
                    HardwareList.Items.Add(item);
                }
            }
            else
            {
                HardwareList.Items.Add("Error creating hardware manager");
            }
        }

        private void UpdateDataTab()
        {
            //Device d = m_drive.Device;
            //Device.CommandStatus st;

            //if (m_leadin_data != null)
            //    return;

            //m_leadin_data = new byte[96 * 150];

            //DataViewer.StartAddress = 0;
            //DataViewer.Data = null;

            //long sector = -150;
            //int length = 150;
            //int count = 5;
            //while (true)
            //{
            //    st = d.ReadSubChannel(4, (uint)sector, 150, ref m_leadin_data);
            //    count--;
            //    if (st == Device.CommandStatus.DeviceFailed && d.GetSenseAsc() == 0x11 && d.GetSenseAscq() == 0 && count > 0)
            //    {
            //        sector += 10;
            //        length -= 10;
            //        continue;
            //    }

            //    if (st == Device.CommandStatus.Success)
            //    {
            //        DataViewer.StartAddress = (int)(-sector);
            //        DataViewer.Data = m_leadin_data;
            //    }
            //    else
            //        m_leadin_data = null;

            //    break;
            //}
        }
    }
}