//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;

namespace Bwg.Isofs
{
    /// <summary>
    /// This class contains a set of constants that are part of the ISO 9660
    /// specification.
    /// </summary>
    public class Constants
    {
        internal const byte BootRecordDescriptor = 0;
        internal const byte PrimaryVolumeDescriptor = 1;
        internal const byte SupplementaryVolumeDescriptor = 2;
        internal const byte VolumePartitionDescriptor = 3;
        internal const byte VolumeDescriptorTerminator = 255;

        /// <summary>
        /// Flags associated with a directory entry
        /// </summary>
        [FlagsAttribute] 
        internal enum DirectoryEntryFileFlags : byte
        {
            HideFile = 0x01,
            Directory = 0x02,
            AssociatedFile = 0x04,
            Record = 0x08,
            Protection = 0x10,
            MultiExtent = 0x80
        } ;

        /// <summary>
        /// Length of the system identifier in characters
        /// </summary>
        public const int SystemIdentifierLength = 32;

        /// <summary>
        /// The length of the volume identifier in characters
        /// </summary>
        public const int VolumeIdentifierLength = 32;

        /// <summary>
        /// The length of the volume set identifer in characters
        /// </summary>
        public const int VolumeSetIdentifierLength = 128;

        /// <summary>
        /// The length of the publisher in characters
        /// </summary>
        public const int PublisherLength = 128;

        /// <summary>
        /// The length of the preparer in characters
        /// </summary>
        public const int PreparerLength = 128;

        /// <summary>
        /// The length of the application identifier in characters
        /// </summary>
        public const int ApplicationIdentifierLength = 128;

        /// <summary>
        /// The length of the copyright file identifier in characters
        /// </summary>
        public const int CopyrightFileIdentifierLength = 37;

        /// <summary>
        /// The length of the abstract file in characters
        /// </summary>
        public const int AbstractFileIdentifierLength = 37;

        /// <summary>
        /// The length of the bibliographic file in characters
        /// </summary>
        public const int BibliographicFileIdentifierLength = 37;
    }
}
