//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;

namespace Bwg.Isofs
{
    class DirectoryEntryDescriptor : InformationControlBlockTask
    {
        private DiskImageDirectory m_dir;

        public DirectoryEntryDescriptor(DiskImageDirectory dir)
        {
            m_dir = dir;
            BlockAddress = m_dir.GetDirectoryLocation(DiskImageDirectory.DirectoryLocationType.UdfFileEntry).Location;
        }

        protected override ICBType GetIcbType()
        {
            return ICBType.Directory;
        }

        protected override short GetStrategy()
        {
            return 4;
        }

        protected override short GetStrategyParameter()
        {
            return 0;
        }

        protected override short GetMaxEntries()
        {
            return 1;
        }

        protected override long GetInformationLength()
        {
            return UDFDirectoryAndFileTask.DirectorySizeInBytes(m_dir);
        }

        protected override int GetContextExtentCount()
        {
            return 1;
        }

        protected override ExtentAd GetContentExtent(int which)
        {
            Debug.Assert(which == 0);

            long loc = m_dir.GetDirectoryLocation(DiskImageDirectory.DirectoryLocationType.UdfFileIdentifierDescriptor).Location;
            return new ExtentAd(loc, (int)GetInformationLength());
        }

        protected override short GetLinkCount()
        {
            short ret = 1;          // The reference to this directory from its parent

            // Add any reference by subdirs
            ret += (short)m_dir.DirectoryCount;
            return ret;
        }

        protected override DateTime AccessDate(DiskImageCreator creator)
        {
            DateTime ret = creator.Settings.UdfSettings.CreationDateTime;

            if (Directory.Exists(m_dir.FileSystemLocation))
            {
                DirectoryInfo info = new DirectoryInfo(m_dir.FileSystemLocation);
                ret = info.LastAccessTime;
            }
            return ret;
        }

        protected override DateTime ModificationDate(DiskImageCreator creator)
        {
            DateTime ret = creator.Settings.UdfSettings.CreationDateTime;

            if (Directory.Exists(m_dir.FileSystemLocation))
            {
                DirectoryInfo info = new DirectoryInfo(m_dir.FileSystemLocation);
                ret = info.LastWriteTime;
            }
            return ret;
        }

        protected override DateTime AttributeDate(DiskImageCreator creator)
        {
            return creator.Settings.UdfSettings.CreationDateTime;
        }
    }
}
