//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics; 

namespace Bwg.Isofs
{
    class ElToritoBootCatalogTask : CreatorTask
    {
        public ElToritoBootImage BootImageTask;

        public ElToritoBootCatalogTask()
        {
            BootImageTask = null;
        }

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            info.CurrentBlock++;
            return true;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            long pos = strm.Position;

            if (!CreateValidationEntry(creator, strm))
                return false;

            if (!CreateInitialEntry(creator, strm))
                return false;

            strm.WriteZeros((uint)(pos + 2048 - strm.Position));

            return true;
        }

        private bool CreateValidationEntry(DiskImageCreator creator, IsoWriteStream strm)
        {
            int sum = 0 ;

            strm.WriteUint8(1);                             // Entry type
            sum += 1 ;

            strm.WriteUint8(creator.Settings.BootPlatformID) ;       // The Platform ID
            sum += (creator.Settings.BootPlatformID << 8);

            strm.WriteUint16Small(0);                       // Mandatory two bytes of zero

            string str = "Arnes Boot Record" ;
            strm.WriteString(Encoding.ASCII, str , 24, (char)0);                  // Found in many bootable disks as the ID string
            int shift = 0;

            foreach(char ch in str)
            {
                byte b = (byte)ch ;
                sum += (b << shift);
                shift = 8 - shift;
            }

            sum += 0x55 ;
            sum += (0xAA << 8);

            while (sum >= 0x10000)
                sum -= 0x10000;

            //
            // sum should be 0x517f for BootPlatformID = 0
            //
            sum = 0x10000 - sum;
            strm.WriteUint16Small((ushort)sum) ;                    // Write checksum
            strm.WriteUint8(0x55);                          // Magic numbers from early IBM PC Days
            strm.WriteUint8(0xAA);                          // Magic numbers from early IBM PC Days

            return true;
        }

        /// <summary>
        /// This method writes the initial/default entry into the file.  This method writes
        /// 16 bytes into the ISO image.
        /// </summary>
        /// <param name="creator">the image creator object</param>
        /// <param name="strm">the stream to write data to</param>
        /// <returns>returns true if the entry was written, false if an error occurred</returns>
        private bool CreateInitialEntry(DiskImageCreator creator, IsoWriteStream strm)
        {
            // We may need to get this from the boot image, I have not figured this out yet
            byte systemtype = 6 ;

            strm.WriteUint8(0x88);                                      // Flag meaning bootable
            strm.WriteUint8((byte)creator.Settings.Emulation);          // No emulation
            strm.WriteUint16Small(creator.Settings.SegmentLoadAddr);    // Default load segment (0x7C0)
            strm.WriteUint8(systemtype);                                // System Type
            strm.WriteUint8(0);                                         // Unused, must be zero

            // Get the boot catalog
            Debug.Assert(BootImageTask != null);

            strm.WriteUint16Small(1);                                   // Sector count
            strm.WriteUint32Small((uint)BootImageTask.BlockAddress);    // Location of the boot image
            strm.WriteZeros(4);

            return true;
        }
    }
}
