//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Bwg.Isofs
{
    class LogicalVolumeDescriptorTask : SequencedUDFDescriptor
    {
        public CreatorTask IntegrityTask;

        public LogicalVolumeDescriptorTask(int seqno) : base(seqno)
        {
            IntegrityTask = null;
        }

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            info.CurrentBlock++;
            return true;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            byte[] data = new byte[0];

            Debug.Assert(IntegrityTask != null);

            AddInt(ref data, VolumeDescriptorSequenceNumber);                           // Volume Descriptor Sequence Number
            AddCharset(ref data, UdfDescriptorTask.OSTACompressedUnicode);              // Descriptor Character Set
            AddDString(ref data, creator.Settings.UdfSettings.VolumeIdentifier, 128, 16);           // Logical Volume Identifier
            AddInt(ref data, 2048);                                                     // Logical Block Size
            AddRegid(ref data, RegIdType.DomainID_1_02_1);                              // Domain Identifier
            AddInt(ref data, 2048);
            AddZeros(ref data, 12);                                                     // Logical Volume Contents Use
            AddInt(ref data, 6);                                                        // Map Table Length
            AddInt(ref data, 1);                                                        // Number Of Partition Maps
            AddRegid(ref data, RegIdType.DeveloperID_1);                                // Implementation Identifier
            AddZeros(ref data, 128);                                                    // TODO: Implementation Use
            AddExtentAD(ref data, IntegrityTask.ExtentAd);                              // Integrity Sequence Extent

            // The partition table (one partition)
            AddByte(ref data, 1);
            AddByte(ref data, 6);
            AddByte(ref data, 1);
            AddByte(ref data, 0);
            AddShort(ref data, 0);
            return WriteDescriptor(strm, UdfDescriptorTask.LogicalVolumeDescriptor, 2, data);
        }
    }
}
