//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Bwg.Logging;

namespace Bwg.Isofs
{
    class PadToBlockTask : CreatorTask
    {
        public enum PadMode
        {
            PadToSpecificBlock,
            PadWithBlockCount
        } ;

        /// <summary>
        /// Either the specific block to pad to, or the block count
        /// </summary>
        public readonly long Block ;

        /// <summary>
        /// The mode, either pad to a sepcific block or for a given count of blocks
        /// </summary>
        public readonly PadMode Mode;

        public PadToBlockTask(PadMode m, long s)
        {
            Mode = m;
            Block = s ;
        }

        public override bool UpdateLayout(DiskImageCreator creator, DiskInfo info)
        {
            bool ret = true;

            if (Mode == PadMode.PadToSpecificBlock)
            {
                if (info.CurrentBlock >= creator.StartBlock + Block)
                {
                    Logger logger = creator.GetLogger();
                    string str = "internal error - invalid layout directive requested";
                    str += "- PadToBlock(" + Block.ToString() + ")";
                    logger.LogMessage(new UserMessage(UserMessage.Category.Error, 0, str));
                    ret = false;
                }
                else
                {
                    info.CurrentBlock = creator.StartBlock + Block;
                }
            }
            else
            {
                info.CurrentBlock += Block;
            }

            return ret;
        }

        public override bool Write(DiskImageCreator creator, DiskInfo info, IsoWriteStream strm)
        {
            byte[] block = new byte[info.BlockSize];

            if (Mode == PadMode.PadToSpecificBlock)
            {
                Debug.Assert((strm.Position % 2048) == 0);
                long curpos = strm.Position / 2048 + creator.StartBlock;
                Debug.Assert(curpos >= 0);

                while (curpos < Block + creator.StartBlock)
                {
                    strm.Write(block);
                    curpos++;
                }
            }
            else
            {
                for (int i = 0; i < Block; i++)
                {
                    strm.Write(block);
                }
            }

            return true;
        }
    }
}
