//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Bwg.Audio.Acm;

namespace Bwg.RipAudio
{
    public partial class MusicLibrarySettingsForm : Form
    {
        #region private member variables
        private AcmManager m_audio_mgr;
        private AcmDriver m_driver;
        private AcmFormatDetails m_format;
        private string m_song_path;
        private bool m_percent_key;
        private bool m_eject_when_done;
        #endregion

        #region constructor
        public MusicLibrarySettingsForm(bool ej, AcmManager mgr, AcmDriver drv, AcmFormatDetails fmt, string curpath)
        {
            InitializeComponent();

            m_audio_mgr = mgr;

            LibraryPath.Text = curpath;
            PopulateAudioDrivers();

            m_song_path = curpath;

            m_eject_when_done = ej;
            EjectWhenDone.Checked = ej;

            SetAudio(drv, fmt);

            m_percent_key = false;
            LibraryPath.KeyDown += new KeyEventHandler(LibraryPath_KeyDown);
            LibraryPath.KeyPress += new KeyPressEventHandler(LibraryPath_KeyPress);
            LibraryPath.MouseUp += new MouseEventHandler(LibraryPath_MouseUp);
            LibraryPath.TextChanged += new EventHandler(LibraryPath_TextChanged);

            TokenList.DoubleClick += new EventHandler(TokenList_DoubleClick);
        }
        #endregion

        #region public properties
        /// <summary>
        /// This property returns the selected driver for audio ripping
        /// </summary>
        public AcmDriver Driver
        {
            get
            {
                return m_driver;
            }
        }

        /// <summary>
        /// This property returns the selected format assocaited with the driver for
        /// audio ripping
        /// </summary>
        public AcmFormatDetails Format
        {
            get
            {
                return m_format;
            }
        }

        /// <summary>
        /// This property retuns the current song path from the form
        /// </summary>
        public string SongPath
        {
            get
            {
                return m_song_path;
            }
        }

        /// <summary>
        /// This property returns true if the disk is to be ejected when done
        /// </summary>
        public bool EjectWhenDoneValue
        {
            get
            {
                return m_eject_when_done;
            }
        }

        #endregion

        #region private methods

        private void TokenList_DoubleClick(object sender, EventArgs args)
        {
            int start = LibraryPath.SelectionStart;
            string token = (string)TokenList.SelectedItem;

            if (token[0] == '@' && start != 0)
            {
                MessageBox.Show("This token and all that begin with an '@' character, must appear at the beginning of "
                                + "the library path and must be the only token with the '@' character used.", 
                                "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (LibraryPath.Text.Length > 1 && LibraryPath.Text[0] == '%' && LibraryPath.Text[1] == '@')
            {
                MessageBox.Show("This token and all that begin with an '@' character, must appear at the beginning of "
                                + "the library path and must be the only token with the '@' character used.", 
                                "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            if (LibraryPath.SelectionLength != 0)
                DeleteText(LibraryPath.SelectionStart, LibraryPath.SelectionLength);

            string str = string.Empty;
            if (start > 0)
                str += LibraryPath.Text.Substring(0, start);

            str += '%';
            str += TokenList.SelectedItem;
            str += '%';

            if (start != LibraryPath.Text.Length)
                str += LibraryPath.Text.Substring(start);

            LibraryPath.Text = str;
            LibraryPath.SelectionStart = start;
            LibraryPath.SelectionLength = 0;
        }

        private void LibraryPath_TextChanged(object sender, EventArgs args)
        {
            m_song_path = LibraryPath.Text;
        }

        private void LibraryPath_KeyDown(object sender, KeyEventArgs args)
        {
            if (args.KeyCode == Keys.D5 && ModifierKeys == Keys.Shift)
                m_percent_key = true;

            if (args.KeyCode == Keys.Left || args.KeyCode == Keys.Up)
            {
                if (LibraryPath.SelectionStart > 0 && LibraryPath.Text[LibraryPath.SelectionStart - 1] != '%')
                {
                    LibraryPath.SelectionStart--;
                    LibraryPath.SelectionLength = 0;
                }
                else if (LibraryPath.SelectionStart != 0)
                {
                    // We are sitting at the end of a token and want to move left, skip the entire token
                    int token = FindTokenIndex(LibraryPath.SelectionStart - 1);
                    LibraryPath.SelectionStart = FindTokenStart(token);
                    LibraryPath.SelectionLength = 0;
                }
                args.SuppressKeyPress = true;
            }
            else if (args.KeyCode == Keys.Right || args.KeyCode == Keys.Down)
            {
                if (LibraryPath.SelectionStart < LibraryPath.Text.Length && LibraryPath.Text[LibraryPath.SelectionStart] != '%')
                {
                    LibraryPath.SelectionStart++;
                    LibraryPath.SelectionLength = 0;
                }
                else if (LibraryPath.SelectionStart < LibraryPath.Text.Length)
                {
                    int token = FindTokenIndex(LibraryPath.SelectionStart + 1);
                    LibraryPath.SelectionStart = FindTokenEnd(token);
                    LibraryPath.SelectionLength = 0;
                }

                args.SuppressKeyPress = true;
            }
            else if (args.KeyCode == Keys.Back)
            {
                if (LibraryPath.SelectionStart > 0 && LibraryPath.Text[LibraryPath.SelectionStart - 1] == '%')
                {
                    // We want to delete the entire token
                    int token = FindTokenIndex(LibraryPath.SelectionStart - 1);
                    int start = FindTokenStart(token);
                    int length = FindTokenEnd(token) - start;
                    DeleteText(start, length);
                    LibraryPath.SelectionStart = start;
                    LibraryPath.SelectionLength = 0;
                    args.SuppressKeyPress = true;
                }
            }
            else if (args.KeyCode == Keys.Delete)
            {
                if (LibraryPath.SelectionStart < LibraryPath.Text.Length && LibraryPath.Text[LibraryPath.SelectionStart] == '%')
                {
                    // We want to delete the entire token
                    int token = FindTokenIndex(LibraryPath.SelectionStart + 1);
                    int start = FindTokenStart(token);
                    int length = FindTokenEnd(token) - start;
                    DeleteText(start, length);
                    LibraryPath.SelectionStart = start;
                    LibraryPath.SelectionLength = 0;
                    args.SuppressKeyPress = true;
                }
            }
            else
            {
                LibraryPath_CheckSelection();
            }
        }

        private void DeleteText(int start, int length)
        {
            string str = string.Empty;

            if (start > 0)
                str += LibraryPath.Text.Substring(0, start);

            if (start + length < LibraryPath.Text.Length)
                str += LibraryPath.Text.Substring(start + length);

            LibraryPath.Text = str;
        }

        private void LibraryPath_KeyPress(object sender, KeyPressEventArgs args)
        {
            if (m_percent_key)
            {
                string str = string.Empty ;
                
                str += "You cannot enter a '%' character into the Library Path. " ;
                str += "If you are trying to enter a token into the path, double click the desired token in the token list. " ;
                str += "If you are trying to enter a '%' character into the path, use the %Percent% token." ;
                MessageBox.Show(str, "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                args.Handled = true;
                m_percent_key = false;
            }
        }

        private void LibraryPath_MouseUp(object sender, MouseEventArgs args)
        {
            LibraryPath_CheckSelection();
        }

        private void LibraryPath_CheckSelection()
        {
            int token;
            int start = -1, length = -1;

            token = FindTokenIndex(LibraryPath.SelectionStart);
            if (token != -1)
                start = FindTokenStart(token);

            token = FindTokenIndex(LibraryPath.SelectionStart + LibraryPath.SelectionLength);
            if (token != -1)
                length = FindTokenEnd(token) - start ;

            if (start != -1)
                LibraryPath.SelectionStart = start;
            if (length != -1)
                LibraryPath.SelectionLength = length;
        }

        private int FindTokenIndex(int pos)
        {
            int index = 0;
            bool intoken = false;
            for (int i = 0; i < LibraryPath.Text.Length; i++)
            {
                if (i == pos && intoken)
                    return index;

                if (LibraryPath.Text[i] == '%')
                {
                    if (intoken)
                    {
                        intoken = false;
                        index++;
                    }
                    else
                        intoken = true;
                }
            }
            return -1;
        }

        private int FindTokenStart(int token)
        {
            bool intoken = false;
            for (int i = 0; i < LibraryPath.Text.Length; i++)
            {
                if (LibraryPath.Text[i] == '%')
                {
                    if (intoken)
                    {
                        intoken = false;
                        token--;
                    }
                    else
                    {
                        if (token == 0)
                            return i;

                        intoken = true;
                    }
                }
            }
            return -1;
        }

        private int FindTokenEnd(int token)
        {
            bool intoken = false;
            for (int i = 0; i < LibraryPath.Text.Length; i++)
            {
                if (LibraryPath.Text[i] == '%')
                {
                    if (intoken)
                    {
                        intoken = false;
                        if (token == 0)
                            return i + 1;

                        token--;
                    }
                    else
                    {
                        intoken = true;
                    }
                }
            }
            return -1;
        }

        private void SetAudio(AcmDriver drv, AcmFormatDetails fmt)
        {
            if (drv != null)
            {
                AudioDriverList.SelectedItem = drv;
                if (fmt != null)
                    AudioFormatList.SelectedItem = fmt;
            }
        }

        private void PopulateAudioDrivers()
        {
            AudioDriverList.Items.Clear();
            foreach (AcmDriver driver in m_audio_mgr.Drivers)
            {
                if (m_audio_mgr.CanConvert(driver, null))
                    AudioDriverList.Items.Add(driver);
            }
            AudioDriverList.SelectedIndex = 0;
        }

        private void AudioDriverList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (AudioDriverList.SelectedItem != null)
            {
                Debug.Assert(AudioDriverList.SelectedItem is AcmDriver);
                m_driver = (AcmDriver)AudioDriverList.SelectedItem;
            }

            AudioFormatList.Items.Clear();

            if (AudioDriverList.SelectedItem is AcmDriver)
            {
                AcmDriver drv = (AcmDriver)AudioDriverList.SelectedItem;
                foreach (AcmFormatDetails fmt in drv.FormatDescriptors)
                {
                    if (m_audio_mgr.CanConvert(drv, null, fmt))
                        AudioFormatList.Items.Add(fmt);
                }

                AudioFormatList.SelectedIndex = 0;
            }
        }

        private void AudioFormatList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (AudioFormatList.SelectedItem != null)
            {
                Debug.Assert(AudioFormatList.SelectedItem is AcmFormatDetails);
                m_format = (AcmFormatDetails)AudioFormatList.SelectedItem;
            }

        }

        private void TokenList_SelectedIndexChanged(object sender, EventArgs e)
        {
            string txt = (string)TokenList.SelectedItem;
            if (txt == "@MyMusic")
            {
                TokenInfo.Text = "This token is replaced with the current location of your 'My Music' folder.";
            }
            else if (txt == "@MyDocuments")
            {
                TokenInfo.Text = "This token is replaced with the current location of your 'My Documents' folder.";
            }
            else if (txt == "@Desktop")
            {
                TokenInfo.Text = "This token is replaced with the current location of your Desktop.";
            }
            else if (txt == "DiscArtist")
            {
                TokenInfo.Text = "This token is replaced with the artist for the disc being ripped";
            }
            else if (txt == "DiscTitle")
            {
                TokenInfo.Text = "This token is replaced with the title for the disc being ripped";
            }
            else if (txt == "TrackArtist")
            {
                TokenInfo.Text = "This token is replaced with the artist for the track being ripped";
            }
            else if (txt == "TrackTitle")
            {
                TokenInfo.Text = "This token is replaced with the title for the track being ripped";
            }
            else if (txt == "TrackNumber")
            {
                TokenInfo.Text = "This token is replaced with the number for the track being ripped";
            }
            else
            {
                TokenInfo.Text = "Unknown Token - Internal Error";
            }
        }

        #endregion

        private void EjectWhenDone_CheckedChanged(object sender, EventArgs e)
        {
            m_eject_when_done = EjectWhenDone.Checked;
        }
    }
}
