//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics ;
using Bwg.Audio.Acm ;

namespace Bwg.RipAudio
{
    public partial class OutputAudioSettings : Form
    {
        /// <summary>
        /// The ACM manager
        /// </summary>
        private AcmManager m_acm ;

        public AcmDriver Driver;
        public AcmFormatDetails Format;

        public OutputAudioSettings(AcmManager mgr, AcmDriver drv, AcmFormatDetails dtls)
        {
            m_acm = mgr ;

            Driver = null;
            Format = null;

            InitializeComponent();
            PopulateAudioDriverList();

            if (drv != null)
            {
                Driver = drv;
                AudioDriverList.SelectedItem = drv;
            }

            if (dtls != null)
            {
                Format = dtls;
                AudioFormatList.SelectedItem = dtls;
            }
        }

        private void PopulateAudioDriverList()
        {
            AudioDriverList.Items.Clear();
            foreach (AcmDriver driver in m_acm.Drivers)
            {
                if (m_acm.CanConvert(driver, null))
                    AudioDriverList.Items.Add(driver);
            }

            AudioDriverList.SelectedIndex = 0;
        }

        private void AudioDriverList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (AudioDriverList.SelectedItem != null)
            {
                Debug.Assert(AudioDriverList.SelectedItem is AcmDriver);
                Driver = (AcmDriver)AudioDriverList.SelectedItem;
            }

            AudioFormatList.Items.Clear() ;

            if (AudioDriverList.SelectedItem is AcmDriver)
            {
                AcmDriver drv = (AcmDriver)AudioDriverList.SelectedItem;
                foreach (AcmFormatDetails fmt in drv.FormatDescriptors)
                {
                    if (m_acm.CanConvert(drv, null, fmt))
                        AudioFormatList.Items.Add(fmt);
                }

                AudioFormatList.SelectedIndex = 0;
            }
        }

        private void AudioFormatList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (AudioFormatList.SelectedItem != null)
            {
                Debug.Assert(AudioFormatList.SelectedItem is AcmFormatDetails);
                Format = (AcmFormatDetails)AudioFormatList.SelectedItem;
            }
        }
    }
}