//
// BwgBurn - CD-R/CD-RW/DVD-R/DVD-RW burning program for Windows XP
// 
// Copyright (C) 2006 by Jack W. Griffin (butchg@comcast.net)
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by 
// the Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program; if not, write to the 
//
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Bwg.Utils
{
    public class StartupSettings
    {
        /// <summary>
        /// The name of the file for the settings
        /// </summary>
        private string m_filename;

        /// <summary>
        /// A map from keys to values
        /// </summary>
        private IDictionary<string, object> m_value_map;

        public StartupSettings(string dbfile)
        {
            m_filename = dbfile;
            m_value_map = new Dictionary<string, object>();
        }

        /// <summary>
        /// Read the settings from a file
        /// </summary>
        /// <returns></returns>
        public bool ReadSettingsFile()
        {
            FileStream strm;

            try
            {
                strm = new FileStream(m_filename, FileMode.Open, FileAccess.ReadWrite);
            }
            catch (Exception)
            {
                return false;
            }

            TextReader rdr = new StreamReader(strm);

            while (true)
            {
                int i = 0;
                string line = rdr.ReadLine();
                if (line == null)
                    break;

                string name = string.Empty ;
                string type = string.Empty ;
                string value = string.Empty;
                while (i < line.Length && line[i] != '.')
                    name += line[i++];

                if (i == line.Length)
                    continue;

                i++;
                while (i < line.Length && line[i] != '=')
                    type += line[i++];

                if (i == line.Length)
                    continue;
                i++;

                while (i < line.Length)
                    value += line[i++];
                value = value.Trim();

                if (type == "bool")
                {
                    string v = value.ToLower();
                    if (v == "true")
                        m_value_map[name] = true;
                    else
                        m_value_map[name] = false;
                }
                else if (type == "int")
                {
                    int num;
                    if (Int32.TryParse(value, out num))
                        m_value_map[name] = num;
                }
                else if (type == "string")
                {
                    i = 1 ;
                    string v = string.Empty;
                    if (value[0] == '\'')
                    {
                        while (i < value.Length && value[i] != '\'')
                            v += value[i++];

                        m_value_map[name] = v;
                    }
                }
            }

            rdr.Close();
            strm.Close();
            return true;
        }

        /// <summary>
        /// Write the settings to a file
        /// </summary>
        /// <returns></returns>
        public bool WriteSettingsFile()
        {
            FileStream strm ;
            StreamWriter wrtr ;

            try
            {
                strm = new FileStream(m_filename, FileMode.Create, FileAccess.ReadWrite);
                wrtr = new StreamWriter(strm);
            }
            catch (Exception ex)
            {
                string str = ex.Message;
                return false;
            }

            foreach (string key in m_value_map.Keys)
            {
                object obj = m_value_map[key];

                if (obj is string)
                {
                    wrtr.WriteLine(key + ".string='" + obj.ToString() + "'");
                }
                else if (obj is bool)
                {
                    wrtr.WriteLine(key + ".bool=" + obj.ToString());
                }
                else if (obj is Int32)
                {
                    wrtr.WriteLine(key + ".int=" + obj.ToString());
                }
                else
                    throw new Exception("unsupported type in settings file");
            }

            wrtr.Close();
            strm.Close();
            return true;
        }

        public object GetValue(string key)
        {
            object value;

            if (!m_value_map.TryGetValue(key, out value))
                return null;

            return value;
        }

        public void SetValue(string key, object value)
        {
            m_value_map[key] = value;
        }
    }
}
