// BwgAudioScanner.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "BwgAudioScanner.h"
#include <malloc.h>
#include <math.h>

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
    return TRUE;
}

static BOOL scanning = FALSE ;
static short lmin, lmax ;				// The minimum and maximum value for the left channel
static short rmin, rmax ;				// The minimum and maximum value for the right channel
static __int64 ltotal ;					// The sum of the left channel squared
static __int64 rtotal ;					// The sum of the right channel squared
static __int64 samples ;				// The number of samples processed since BwgBeginScan() was called last
static int intervalsize ;				// The number of samples per interval for RMS calculation
static int samplessofar ;				// The number of samples processed so for for this interval
static int measused = 0 ;				// The number of measurements used in the array
static int measalloc = 0 ;				// The number of measurements alloced in the array
static int *measurements ;			// The measurements

//
// Note that the interval is in samples.  To get the 50 ms 
//
BWGAUDIOSCANNER_API unsigned int BwgBeginScan(int interval)
{
	lmin = 0 ;
	lmax = 0 ;
	rmin = 0 ;
	rmax = 0 ;
	ltotal = 0 ;
	rtotal = 0 ;
	samples = 0 ;

	if (measused > 0)
	{
		free(measurements) ;
		measused = 0 ;
		measalloc = 0 ;
	}

	samplessofar = 0 ;
	intervalsize = interval ;
	scanning = TRUE ;

	return 1 ;
}

static void
AddSample(__int64 sample)
{
	if (measused == measalloc)
	{
		if (measused == 0)
		{
			measalloc = 64 ;
			measurements = (int *)malloc(sizeof(int) * measalloc) ;
		}
		else
		{
			measalloc *= 2 ;
			measurements = (int *)realloc(measurements, sizeof(int) * measalloc) ;
		}
	}

	double d = 10 * log10((double)sample + 10e-10) ;
	measurements[measused++] = (int)d ;
}

// This is an example of an exported function.
BWGAUDIOSCANNER_API unsigned int BwgScanBlock(short *data_p, int count, short &pmin, short &pmax)
{
	pmin = 0 ;
	pmax = 0 ;

	if (!scanning)
		return 0 ;

	if (count & 0x03)
		return 0 ;

	count >>= 2 ;		// Divide the count by four
	while (count-- > 0)
	{
		if (*data_p < lmin)
			lmin = *data_p ;
		if (*data_p > lmax)
			lmax = *data_p ;

		ltotal += *data_p * *data_p;

		if (*data_p > pmax)
			pmax = *data_p ;
		if (*data_p < pmin)
			pmin = *data_p ;

		data_p++ ;

		if (*data_p < rmin)
			rmin = *data_p ;
		if (*data_p > rmax)
			rmax = *data_p ;
		rtotal += *data_p * *data_p;
		data_p++ ;

		if (*data_p > pmax)
			pmax = *data_p ;
		if (*data_p < pmin)
			pmin = *data_p ;

		samples++ ;
		samplessofar++ ;

		if (samplessofar == intervalsize)
		{
			// Combine the two channels
			AddSample((ltotal + rtotal) / intervalsize) ;

			rtotal = 0 ;
			ltotal = 0 ;
			samplessofar = 0 ;
		}
	}
	return 1 ;
}

BWGAUDIOSCANNER_API unsigned int BwgEndScan()
{
	if (!scanning)
		return 0 ;

	scanning = 0 ;
	return 1 ;
}

BWGAUDIOSCANNER_API unsigned int BwgGetPeakValues(short *dlmin, short *dlmax, short *drmin, short *drmax)
{
	if (scanning)
		return 0 ;

	*dlmin = lmin ;
	*dlmax = lmax ;
	*drmin = rmin ;
	*drmax = rmax ;

	return 1 ;
}

BWGAUDIOSCANNER_API unsigned int BwgFreeScanResources()
{
	if (scanning)
		return 0 ;

	if (measused > 0)
	{
		free(measurements) ;
		measused = 0 ;
		measalloc = 0 ;
	}

	return 1 ;
}


BWGAUDIOSCANNER_API unsigned int BwgGetAverageValues(short *lavg, short *ravg)
{
	if (scanning)
		return 0 ;

	*lavg = (short)(ltotal / samples) ;
	*ravg = (short)(rtotal / samples) ;

	return 1 ;
}

BWGAUDIOSCANNER_API BOOL BwgCheckZero(short *data_p, int count)
{
	while (count-- > 0)
	{
		if (*data_p++ != 0)
			return FALSE ;
	}

	return TRUE ;
}
